@@ -187,9 +187,13 @@ class SurfingEntrance(object):
         print('Step 3. Done update derived')
 
         print('Step 4. Start update view')
-        view_failed_tasks = self.update_view(yesterday)
-        if len(view_failed_tasks) > 0:
-            failed_tasks['view'] = view_failed_tasks
+        try:
+            view_failed_tasks = self.update_view(yesterday)
+        except Exception:
+            failed_tasks['view'] = ['fatal error']
+        else:
+            if len(view_failed_tasks) > 0:
+                failed_tasks['view'] = view_failed_tasks
         print('Step 4. Done update view')
 
         print('Done data update')
@@ -208,6 +212,8 @@ class SurfingEntrance(object):
         print('dump tables and dfs done')
         print('start to update stock factor')
         self.update_stock_factor(yesterday, wechat_bot)
+        print('start to update fund factor')
+        self.update_fund_factor(yesterday, wechat_bot)
         print('update stock factor done')
         self.check_refinancing(yesterday, wechat_bot)
         self.check_refinancing_impl(yesterday, wechat_bot)
@@ -475,6 +481,36 @@ class SurfingEntrance(object):
             wechat_bot = WechatBot()
         wechat_bot.send_update_stock_factor_result(yesterday, failed_factors)
 
+    def update_fund_factor(self, yesterday=None, wechat_bot=None):
+        import pandas as pd
+        from .stock.factor.api import FundFactorUpdater
+        from .data.api.basic import BasicDataApi
+        
+        if yesterday is None:
+            yesterday = self._get_yesterday()
+
+        # 获取下一个交易日
+        trading_day_df = BasicDataApi().get_trading_day_list(start_date=yesterday)
+        if trading_day_df.shape[0] <= 1:
+            print(f'get trading days start with {yesterday} failed')
+            failed_factors: List[str] = ['get_trading_day for fund factor updating']
+        else:
+            next_trading_day = trading_day_df.iloc[1, :].datetime
+            print(f'got next trading day {next_trading_day}')
+            end_date_dt = pd.to_datetime(yesterday, infer_datetime_format=True).date()
+            next_trading_dt = pd.to_datetime(next_trading_day, infer_datetime_format=True).date()
+            if end_date_dt.weekday() < next_trading_dt.weekday() and next_trading_dt < end_date_dt + datetime.timedelta(weeks=1):
+                # 表明本周后边还有交易日，今天不需要更新
+                print(f'fund factor only update on the last day of week, not today {end_date_dt}')
+                return
+            else:
+                failed_factors: List[str] = FundFactorUpdater.do_update_every_week()
+            
+        if wechat_bot is None:
+            from .util.wechat_bot import WechatBot
+            wechat_bot = WechatBot()
+        wechat_bot.send_update_stock_factor_result(yesterday, failed_factors)
+
     def check_refinancing(self, yesterday=None, wechat_bot=None):
         from .data.api.raw import RawDataApi
 
@@ -486,19 +522,23 @@ class SurfingEntrance(object):
             return
 
         df = df[df.issue_object.notna()]
-        # TODO: 目前key是hard code的
-        key = '高瓴'
-        df = df[df.issue_object.str.contains(key)]
-        if df.empty:
-            return
-        stock_info = RawDataApi().get_em_stock_info(stock_list=df.stock_id.to_list())
-        if stock_info is None:
-            return
 
         if wechat_bot is None:
             from .util.wechat_bot import WechatBot
             wechat_bot = WechatBot()
-        wechat_bot.send_refinancing_key_info(yesterday, stock_info.set_index('stock_id')['name'].to_dict(), key)
+
+        # TODO: 目前这里是hard code的
+        obj_keys = {'高瓴': '^.*高瓴|天津礼仁|珠海赫成.*$'}
+        for key, obj in obj_keys.items():
+            df_one = df[df.issue_object.str.contains(obj)]
+            if df_one.empty:
+                continue
+            stock_list = list(df_one.stock_id.unique())
+            stock_info = RawDataApi().get_em_stock_info(stock_list=stock_list)
+            if stock_info is None:
+                continue
+
+            wechat_bot.send_refinancing_key_info(yesterday, stock_info.set_index('stock_id')['name'].to_dict(), key)
 
     def check_refinancing_impl(self, yesterday=None, wechat_bot=None):
         from .data.api.raw import RawDataApi
@@ -511,25 +551,29 @@ class SurfingEntrance(object):
             return
 
         df = df[df.issue_object.notna()]
-        # TODO: 目前key是hard code的
-        key = '高瓴'
-        df = df[df.issue_object.str.contains(key)]
-        if df.empty:
-            return
-        stock_info = RawDataApi().get_em_stock_info(stock_list=df.stock_id.to_list())
-        if stock_info is None:
-            return
-        stock_price = RawDataApi().get_em_stock_price(start_date=yesterday, end_date=yesterday, stock_list=df.stock_id.to_list(), columns=['close'])
-        if stock_price is None:
-            return
-        stock_info = stock_info.set_index('stock_id')[['name']].join([stock_price.drop(columns='datetime').set_index('stock_id'), df.set_index('stock_id').issue_price], how='outer')
-        if stock_info.empty:
-            return
 
         if wechat_bot is None:
             from .util.wechat_bot import WechatBot
             wechat_bot = WechatBot()
-        wechat_bot.send_refinancing_impl_key_info(yesterday, stock_info.to_markdown(), key)
+
+        # TODO: 目前这里是hard code的
+        obj_keys = {'高瓴': '^.*高瓴|天津礼仁|珠海赫成.*$'}
+        for key, obj in obj_keys.items():
+            df_one = df[df.issue_object.str.contains(obj)]
+            if df_one.empty:
+                continue
+            stock_list = list(df_one.stock_id.unique())
+            stock_info = RawDataApi().get_em_stock_info(stock_list=stock_list)
+            if stock_info is None:
+                continue
+            stock_price = RawDataApi().get_em_stock_price(start_date=yesterday, end_date=yesterday, stock_list=stock_list, columns=['close'])
+            if stock_price is None:
+                continue
+            stock_info = stock_info.set_index('stock_id')[['name']].join([stock_price.drop(columns='datetime').set_index('stock_id'), df_one.set_index('stock_id').issue_price], how='outer')
+            if stock_info.empty:
+                continue
+
+            wechat_bot.send_refinancing_impl_key_info(yesterday, stock_info.to_markdown(), key)
 
 
 if __name__ == "__main__":
