@@ -129,6 +129,19 @@ class IndClassType(enum.IntEnum):
     SWI3 = 3  # 申万三级行业指数
 
 
+# 持仓资产类型
+class HoldingAssetType(enum.IntEnum):
+    MUTUAL = 1  # 公募基金
+    HEDGE = 2  # 私募基金
+
+
+# 资金状态
+class FundStatus(enum.IntEnum):
+    IN_TRANSIT = 1  # 在途
+    DONE = 2  # 完成
+    INSENTIVE_DONE = 3  # 业绩计提扣除份额完成
+
+
 class ExchangeID(ReadableEnum):
     NOT_AVAILABLE = 0
     SSE = 1
