@@ -32,6 +32,18 @@ class BasicDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {TradingDayList.__tablename__}')
 
+    def get_last_week_last_trade_dt(self):
+        today = datetime.datetime.today().date()
+        start_date = today - datetime.timedelta(days=15)
+        df = self.get_trading_day_list(start_date=start_date)
+        df.loc[:,'weekday'] = df.datetime.map(lambda x : x.weekday())
+        td = pd.to_datetime(df.datetime)
+        td = [_.week for _ in td]
+        df.loc[:,'week_num'] = td
+        last_week_num = df.week_num.values[-1]
+        last_week_dt = df[df.week_num != last_week_num].datetime.values[-1]
+        return last_week_dt
+
     def is_today_trading_date(self):
         today = datetime.datetime.now().date()
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -130,12 +142,16 @@ class BasicDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {SectorFunds.__tablename__}')
 
-    def get_fund_info(self):
+    def get_fund_info(self, fund_list: Tuple[str] = ()):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
                         FundInfo
                     )
+                if fund_list:
+                    query = query.filter(
+                        FundInfo.fund_id.in_(fund_list),
+                    )
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
 
@@ -814,18 +830,18 @@ class BasicDataApi(metaclass=Singleton):
     def get_fund_hold_stock(self, dt=None) -> pd.DataFrame:
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
-                if not dt:
-                    dt = db_session.query(
-                        func.max(FundHoldStock.datetime)
-                    ).one_or_none()
-                    if dt:
-                        dt = dt[0]
-
                 query = db_session.query(
                     FundHoldStock
-                ).filter(
-                    FundHoldStock.datetime == dt
                 )
+                if dt:
+                    query = query.filter(
+                        FundHoldStock.datetime == dt
+                    )
+                else:
+                    query = query.filter(
+                        FundHoldStock.datetime > datetime.datetime.now() - datetime.timedelta(days=180)
+                    )
+
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get time range data <err_msg> {e} from {FundHoldStock.__tablename__}')
@@ -837,7 +853,9 @@ class BasicDataApi(metaclass=Singleton):
                 if not dt:
                     dt = db_session.query(
                         func.max(FundHoldStock.datetime)
-                    ).one_or_none()
+                    ).filter(
+                    FundHoldStock.fund_id.in_(fund_list),
+                ).one_or_none()
                     if dt:
                         dt = dt[0]
 
@@ -858,10 +876,11 @@ class BasicDataApi(metaclass=Singleton):
                 if not dt:
                     dt = db_session.query(
                         func.max(FundHoldBond.datetime)
-                    ).one_or_none()
+                    ).filter(
+                    FundHoldBond.fund_id.in_(fund_list),
+                ).one_or_none()
                     if dt:
                         dt = dt[0]
-
                 query = db_session.query(
                     FundHoldBond
                 ).filter(
@@ -878,8 +897,10 @@ class BasicDataApi(metaclass=Singleton):
             try:
                 if not dt:
                     dt = db_session.query(
-                        func.max(FundHoldBond.datetime)
-                    ).one_or_none()
+                        func.max(FundHoldAsset.datetime)
+                    ).filter(
+                    FundHoldAsset.fund_id.in_(fund_list),
+                ).one_or_none()
                     if dt:
                         dt = dt[0]
 
@@ -1042,6 +1063,34 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'failed to get_hedge_fund_nav <err_msg> {e} from {HedgeFundNAV.__tablename__}')
 
 
+    def delete_hedge_fund_nav(self, fund_id_to_delete: str, *, date_list: Tuple[datetime.date] = (), start_date: str = '', end_date: str = '') -> bool:
+        with BasicDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    HedgeFundNAV
+                ).filter(
+                    HedgeFundNAV.fund_id == fund_id_to_delete,
+                )
+                if date_list:
+                    query = query.filter(
+                        HedgeFundNAV.datetime.in_(date_list),
+                    )
+                if start_date:
+                    query = query.filter(
+                        HedgeFundNAV.datetime >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        HedgeFundNAV.datetime <= end_date,
+                    )
+                query.delete(synchronize_session=False)
+                session.commit()
+                return True
+            except Exception as e:
+                print(f'failed to delete data <err_msg> {e} from {HedgeFundNAV.__tablename__}')
+                return False
+
+
     def get_fof_info(self, fof_id_list: Tuple[str] = ()):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
