@@ -619,8 +619,9 @@ class BasicDataPart1:
                 return False
             # 将场内变为场外的
             df['CODES'] = df.CODES.transform(lambda x: x.split('.')[0] + '.OF' if '.OF' not in x else x)
+            df = df.sort_values(by=['DATES', 'CODES']).drop_duplicates(subset=['CODES'], keep='last')
             # pivot on all remaining columns
-            df = df.pivot(index='DATES', columns='CODES').fillna(method='ffill')
+            df = df.pivot(index='DATES', columns='CODES').ffill()
             df = df.iloc[[-1], :].stack().reset_index()
             df['fund_id'] = df[['DATES', 'CODES']].apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
             df = df[df.fund_id.notna()]
