@@ -103,7 +103,7 @@ class BasicFundRet(BasicDataPart1):
         total_ret = self.fund_nav_fill.iloc[-1] / self.fund_nav_fill.iloc[0]
         self.ret_res['to_date_ret'] = (total_ret - 1)
         self.ret_res['annual_ret'] = self.get_annual_ret(total_ret)
-        self.ret_res['vol'] = self.fund_nav.pct_change(1).std(ddof=1)
+        self.ret_res['vol'] = self.fund_nav.pct_change(1).std(ddof=1) * np.sqrt(242)
         annual_vol = self.get_annual_vol()
         self.ret_res['sharpe_ratio'] = ((self.ret_res['annual_ret'] - self.RISK_FEE_RATE) / annual_vol)
         self.ret_res['mdd'] = BasicFundRet.get_mdd(self.fund_nav)
