@@ -1,7 +1,7 @@
 import numpy as np
 import pandas as pd
 from .raw_data_helper import RawDataHelper
-from ...view.raw_models import OSFundInfo, OSFundNav, OSFundBenchmark, OSStockInfo, OSCorpBondInfo, OSGovtBondInfo, OSComdtyInfo, OSIndexFutureInfo, OSMtgeBondInfo, EmOverseaStockFinFac
+from ...view.raw_models import OSFundInfo, OSFundNav, OSFundBenchmark, OSStockInfo, OSCorpBondInfo, OSGovtBondInfo, OSComdtyInfo, OSIndexFutureInfo, OSMtgeBondInfo, EmOverseaStockFinFac, OSFundBenchmarks
 
 # df.to_sql('em_oversea_stock_fin_fac', RawDatabaseConnector().get_engine(), index=False, if_exists='append')
 
@@ -189,4 +189,145 @@ class OtherRawDataDownloader:
             print(data_row)
         '''
         df= df.replace('——',np.nan)
-        self._data_helper._upload_raw(df, EmOverseaStockFinFac.__table__.name)
\ No newline at end of file
+        self._data_helper._upload_raw(df, EmOverseaStockFinFac.__table__.name)
+
+
+    def benchmark_process(self):
+        #### 修补逻辑  
+        ##### 1. bloomberg 1
+        ##### 2. morning star
+        ##### 3. bloomberg 3
+        ##### 4. 人工
+        folder = '/Users/huangkejia/Downloads/Data/'
+        excel = 'Benchmark-210118-unlink.xlsx'
+        fund_benchmark_total = pd.read_excel(folder+excel,sheet_name='Benchmark')
+        fund_benchmark_total = fund_benchmark_total.replace('#N/A Field Not Applicable',np.nan).replace('#N/A Invalid Security',np.nan).replace(0,np.nan).drop(columns=['Benchmark_Souce_Factsheet','筛选原因'])
+        
+        # 无净值基准
+        invaild_index_id = ['PCY US Equity']
+        for i in invaild_index_id:
+            fund_benchmark_total = fund_benchmark_total.replace(i,np.nan)
+        
+        # 替换列名
+        dic = {
+            '基金名称':'desc_name',
+            '产品编号':'codes',
+            'ISIN 编号':'isin_code',
+            '行业-一级分类':'industry_1',
+            '行业-二级分类':'industry_2',
+            '地域侧重':'area',
+            '行业分类_Morningstar':'morning_star_group',
+            '筛选1':'benchmark_artificial',
+            'Benchmark_Souce_Bloomberg Primary':'benchmark_1',
+            'Benchmark_Souce_Bloomberg Secondary':'benchmark_2',
+            'Benchmark_Souce_Bloomberg Third':'benchmark_3',
+            'Benchmark_Souce_Morningstar':'benchmark_ms',
+        }
+        fund_benchmark_total = fund_benchmark_total.rename(columns=dic)
+        
+        # 人工补全基金基准
+        artificial_list = [{
+            'isin_code':'LU1694771590',
+            'morning_star_benchmark':'MSCI World/Information Tech NR USD',
+        },
+        {
+            'isin_code':'LU1529780063',
+            'morning_star_benchmark':'MSCI World/Information Tech NR USD',
+        },
+        {
+            'isin_code':'HK0000179314',
+            'morning_star_benchmark':'Cat 50%Barclays US Agg TR&50%FTSE Wld TR',
+        },
+        {
+            'isin_code':'IE00BG85LS38',
+            'morning_star_benchmark':'BBgBarc US Agg Bond TR USD',
+        },
+        {
+            'isin_code':'IE00BDB5PG57',
+            'morning_star_benchmark':'BBgBarc US Agg Bond TR USD',
+        },
+        {
+            'isin_code':'IE00BLP5S353',
+            'morning_star_benchmark':'MSCI World NR USD',
+        },
+        ]
+        fund_benchmark_total = fund_benchmark_total.set_index('isin_code')
+        for i in artificial_list:
+            fund_benchmark_total.loc[i['isin_code'],'benchmark_ms'] = i['morning_star_benchmark']
+        fund_benchmark_total = fund_benchmark_total.reset_index()
+
+        # 分组设置基金基准
+        fund_benchmark_total.loc[:,'benchmark_final_mix'] = None 
+        df_1 = fund_benchmark_total[~fund_benchmark_total['benchmark_1'].isnull()].copy()
+        df_1['benchmark_final_mix'] = df_1['benchmark_1']
+
+        df_2 = fund_benchmark_total[(fund_benchmark_total['benchmark_1'].isnull())
+                                &(~fund_benchmark_total['benchmark_ms'].isnull())].copy()
+        df_2['benchmark_final_mix'] = df_2['benchmark_ms']
+
+        df_3 = fund_benchmark_total[(fund_benchmark_total['benchmark_1'].isnull())
+                                &(fund_benchmark_total['benchmark_ms'].isnull())
+                                &(~fund_benchmark_total['benchmark_3'].isnull())].copy()
+        df_3['benchmark_final_mix'] = df_3['benchmark_3']
+
+        df_4 = fund_benchmark_total[(fund_benchmark_total['benchmark_1'].isnull())
+                                &(fund_benchmark_total['benchmark_ms'].isnull())
+                                &(fund_benchmark_total['benchmark_3'].isnull())].copy()
+        df_4['benchmark_final_mix'] = df_4['benchmark_artificial']
+        df = pd.concat([df_1,df_2,df_3,df_4])
+
+        # 替换用bloomberg代码替换晨星代码
+        folder = '/Users/huangkejia/Downloads/Data/'
+        excel = 'Benchmark-210118-unlink.xlsx'
+        ms_data = pd.read_excel(folder+excel,sheet_name='Morning star对应BBG Ticker')
+        ms_ticker_to_bb_ticker_dic = ms_data.set_index('Category Index').to_dict()['Bloomberg code ']
+        df.loc[:,'benchmark_final_bb_only'] = df.benchmark_final_mix.replace(ms_ticker_to_bb_ticker_dic)
+
+        # 去掉index_id 代码后缀 '_index' '_Index' .. 
+        df.loc[:,'benchmark_final'] =  df.benchmark_final_bb_only.map(lambda x:x.replace(' Index','').replace(' index','').replace(' INDEX','').replace(' iNDEX',''))
+
+        # 部分指数是jpm 没有基金代码 人工选取替换
+        replace_dict = {
+            'JPMGGLBL':'BGAGTRUU',
+            'JPMGEUTR':'LEGVTREU',
+            'JACINGTR':'I02885JP',
+            'JBCD103':'JPEIGLBL',
+            'JPMGBRUS':'HEC0',
+            'NRIEEED':'M1EU4000',
+            'JESGEMHE':'BEHGTRUU',
+            'DLJLVAL':'BGAGTRUU',
+            'JGENBAUU':'I02859JP',
+            'JPMGGLBP':'LG30TRUU',
+            'M1CXKAA':'SPEMAUT',
+            'JACIIGPR':'I02859JP',
+            'JPGCCOMP':'BEHGTRUU',
+            'JACICOTR':'I02859JP',
+            'JGENVUUG':'BEHGTRUU',
+            'JBCDCOMP':'BEHGTRUU',
+            'JPEGHECP':'BEHGTRUU',
+            'JPGCCPPI':'BEHGTRUU',
+        }
+        df.benchmark_final = df.benchmark_final.replace(replace_dict)
+
+        # 部分指数没有净值 人工替换
+        replace_dic = {
+            'HEC0':'LP02OAS',
+            'MSAUCAPF':'MSAPUSDN',
+            'HW00':'LG30TRUU',
+            'H4DC':'LG30TRUU',
+            'E0A0':'BEHGTRUU',
+            'BVA0':'LU13TRUU',
+            'HUC0':'LF98TRUU',
+            'EBRL':'LECPTREU',
+            'FTGCY':'M1WO0MT',
+            'C0C0':'LUACTRUU',
+            'HW0C':'LG30TRUU',
+            'JUC0':'LF98TRUU',
+            'MSAUIA':'MXIN',
+            'HW4C':'LG30TRUU',
+            'G0O1':'LEGATRUU',
+            'H0A0':'LG30TRUU',   
+        }
+        df.benchmark_final = df.benchmark_final.replace(replace_dic)
+        #df.to_sql('oversea_fund_benchmarks', RawDatabaseConnector().get_engine(), index=False, if_exists='append')
+        self._data_helper._upload_raw(df, OSFundBenchmarks.__table__.name)
\ No newline at end of file
