@@ -635,7 +635,7 @@ class FundDailyCollectionProcessor(object):
     def process(self):
         failed_tasks = []
         if not self.compute_daily_collection():
-            failed_tasks.append('compute_daily_collection')
+            failed_tasks.append('compute_fund_daily_collection')
         return failed_tasks
 
 
