@@ -1,18 +1,32 @@
 
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Union
+from collections import defaultdict
 import re
 import datetime
+import json
+import io
 
 import numpy as np
 import pandas as pd
 
 from ...util.singleton import Singleton
+from ...util.wechat_bot import WechatBot
 from ...constant import FundStatus, HoldingAssetType
 from ..api.basic import BasicDataApi
 from ..api.derived import DerivedDataApi
 from ..view.basic_models import FOFManually, HedgeFundNAV
-from ..view.derived_models import FOFNav
+from ..view.derived_models import FOFNav, FOFPosition
 from ..wrapper.mysql import BasicDatabaseConnector, DerivedDatabaseConnector
+from ..nav_reader.hedge_fund_nav_reader import HedgeFundNAVReader
+
+
+# def IncentiveFeeMode(enum.IntEnum):
+#     ASSET_HW = 1  # 基金资产高水位方法
+#     ASSET_HW_WITH_EXTRA = 2  # 基金资产高水位方法
+#     SINGLE_CUSTOM_HW = 3  # 基金资产高水位）赎回时补充计提法
+#     SINGLE_CUSTOM_HW_FIXED_WITH_EXTRA = 4  # （单客户高水位）固定时点扣减份额和赎回时补充计提法
+#     SINGLE_CUSTOM_HW_DIVIDEND_WITH_EXTRA = 5  # （单客户高水位）分红时计提和赎回时补充计提法
+#     SINGLE_CUSTOM_HW_OPEN_DAY = 6   # （单客户高水位）单客户开放日扣减净值计提法
 
 
 class FOFDataManager(metaclass=Singleton):
@@ -38,11 +52,27 @@ class FOFDataManager(metaclass=Singleton):
         self._fund_pos: Optional[pd.DataFrame] = None
         self._manually: Optional[pd.DataFrame] = None
         self._fof_nav: Optional[pd.DataFrame] = None
+        self._fof_position: Optional[pd.DataFrame] = None
+
+        self._wechat_bot = WechatBot()
 
     def _get_days_this_year_for_fee(self, the_date: datetime.date) -> int:
         '''计算今年一共有多少天'''
         return pd.Timestamp(year=the_date.year, month=12, day=31).dayofyear
 
+    @staticmethod
+    def _do_calc_v_net_value(nav: Union[pd.Series, pd.DataFrame, float], acc_nav: pd.Series, init_water_line: pd.Series, incentive_fee_ratio: Union[pd.Series, float], decimals: Union[pd.Series, float]) -> pd.Series:
+        # 盈利
+        excess_ret = acc_nav - init_water_line
+        # 盈 或 亏
+        earn_con = (excess_ret > 0).astype('int')
+        # 费
+        pay_mng_fee = excess_ret * earn_con * incentive_fee_ratio
+        # 净值
+        nav -= pay_mng_fee.round(decimals)
+        nav = nav.round(decimals)
+        return nav
+
     @staticmethod
     def _calc_virtual_net_value():
         '''计算虚拟净值'''
@@ -55,30 +85,31 @@ class FOFDataManager(metaclass=Singleton):
         df = df[df.fund_id.isin(fund_info.index.array)]
         df = df.sort_values(by=['fund_id', 'datetime', 'insert_time']).drop_duplicates(subset=['fund_id', 'datetime'], keep='last')
 
+        asset_allocation = FOFDataManager.get_fof_asset_allocation()
+        asset_allocation = asset_allocation[asset_allocation.status == FundStatus.INSENTIVE_DONE]
+
         trading_day_list = BasicDataApi().get_trading_day_list(start_date=df.datetime.sort_values().array[0], end_date=datetime.datetime.now().date())
-        df = df.pivot(index='datetime', columns='fund_id', values='net_asset_value').reindex(trading_day_list.datetime).ffill()
-        # 盈利
-        excess_ret = (df - fund_info.water_line)
-        # 盈 或 亏
-        earn_con = (excess_ret > 0) * 1
-        # 费
-        pay_mng_fee = excess_ret * earn_con * fund_info.incentive_fee_ratio
-        # 净值
-        df -= pay_mng_fee
-        df = df.round(fund_info.v_nav_decimals)
-        return df
+        temp = df.pivot(index='datetime', columns='fund_id').reindex(trading_day_list.datetime).ffill()
+        nav, acc_nav = temp['net_asset_value'], temp['acc_unit_value']
+        init_water_line = fund_info.water_line.to_frame('init_water_line')
+        init_water_line['datetime'] = nav.index.array[0]
+        init_water_line = init_water_line.reset_index().pivot(index='datetime', columns='fund_id', values='init_water_line').reindex(index=nav.index)
+        for row in asset_allocation.itertuples(index=False):
+            init_water_line.loc[row.datetime, row.fund_id] = nav.loc[nav.index < row.datetime].iloc[-2, :][row.fund_id]
+        init_water_line = init_water_line.ffill()
+
         # print(f'do not support the incentive fee mode: {self._INCENTIVE_FEE_MODE}')
+        return FOFDataManager._do_calc_v_net_value(nav, acc_nav, init_water_line, fund_info.incentive_fee_ratio, fund_info.v_nav_decimals)
 
     def init(self):
         # 获取fof基本信息
         fof_info: Optional[pd.DataFrame] = FOFDataManager.get_fof_info([FOFDataManager._FOF_ID])
         assert fof_info is not None, f'get fof info for {FOFDataManager._FOF_ID} failed'
 
-        fof_info = fof_info.sort_values(by=['fof_id', 'datetime']).iloc[-1]
         self._MANAGEMENT_FEE_PER_YEAR = fof_info.management_fee
         self._CUSTODIAN_FEE_PER_YEAR = fof_info.custodian_fee
         self._ADMIN_SERVICE_FEE_PER_YEAR = fof_info.administrative_fee
-        self._DEPOSIT_INTEREST_PER_YEAR = fof_info.current_depoist_rate
+        self._DEPOSIT_INTEREST_PER_YEAR = fof_info.current_deposit_rate
         self._SUBSCRIPTION_FEE = fof_info.subscription_fee
         self._ESTABLISHED_DATE = fof_info.established_date
         self._INCENTIVE_FEE_MODE = fof_info.incentive_fee_mode
@@ -123,7 +154,12 @@ class FOFDataManager(metaclass=Singleton):
         # 获取私募基金净值数据
         hedge_fund_nav = FOFDataManager.get_hedge_fund_nav()
         hedge_fund_nav = hedge_fund_nav.sort_values(by=['fund_id', 'datetime', 'insert_time']).drop_duplicates(subset=['fund_id', 'datetime'], keep='last')
-        self._hedge_nav = hedge_fund_nav.pivot(index='datetime', columns='fund_id', values='v_net_value').reindex(index=self._date_list).ffill()
+        self._hedge_nav = hedge_fund_nav.pivot(index='datetime', columns='fund_id', values='v_net_value').reindex(index=self._date_list)
+
+        # 我们自己算一下虚拟净值 然后拿它对_hedge_nav查缺补漏
+        v_nav_calcd = self._calc_virtual_net_value()
+        # 最后再ffill
+        self._hedge_nav = self._hedge_nav.combine_first(v_nav_calcd.reindex(index=self._date_list)).ffill()
 
         # 获取人工手工校正信息
         manually = BasicDataApi().get_fof_manually([FOFDataManager._FOF_ID])
@@ -143,7 +179,7 @@ class FOFDataManager(metaclass=Singleton):
         errors = errors.rename(columns={'每日管理费误差': 'management_fee_error', '每日行政服务费误差': 'admin_service_fee_error', '每日托管费误差': 'custodian_fee_error'})
         errors = errors[errors.notna().any(axis=1)].set_index('datetime').sort_index()
 
-        manually = BasicDataApi().get_fof_manually([FOFDataManager._FOF_ID]).drop(columns='_update_time')
+        manually = BasicDataApi().get_fof_manually([FOFDataManager._FOF_ID]).drop(columns=['update_time', 'create_time', 'is_deleted'])
         manually = manually.set_index('datetime')
         manually = manually.combine_first(errors)
         manually['fof_id'] = FOFDataManager._FOF_ID
@@ -282,6 +318,7 @@ class FOFDataManager(metaclass=Singleton):
             misc_fees_list = pd.Series(dtype='float64', name='misc_fees')
             misc_amount_list = pd.Series(dtype='float64', name='misc_amount')
             deposit_interest_list = pd.Series(dtype='float64', name='deposit_interest')
+            positions_list = pd.Series(dtype='float64', name='positions')
             trades_in_transit = []
             deposit_in_transit = []
             total_cash = 0
@@ -325,6 +362,8 @@ class FOFDataManager(metaclass=Singleton):
                 finally:
                     total_cash = round(total_cash, 2)
                     share_increased = round(share_increased, 2)
+                if share_increased > 0:
+                    print(f'{self._FOF_ID} share changed (date){date} (amount){total_amount} (share){share_increased}')
 
                 # 这个日期之后才正式成立，所以在此之前都不需要处理后续步骤
                 if date < self._ESTABLISHED_DATE:
@@ -361,7 +400,7 @@ class FOFDataManager(metaclass=Singleton):
                         if not pd.isnull(row.share):
                             # 业绩报酬计提扣除份额完成
                             if row.status == FundStatus.INSENTIVE_DONE:
-                                self._hedge_pos.loc[self._hedge_pos.index>=date, row.fund_id] -= row.share
+                                self._hedge_pos.loc[self._hedge_pos.index >= date, row.fund_id] -= row.share
                                 # 重刷hedge_fund_mv
                                 hedge_fund_mv = self._get_hedge_mv()
                     assert total_cash >= cash_in_transit, f'no enough cash to buy asset!! (date){date} (total cash){total_cash} (cash in transit){cash_in_transit}'
@@ -409,15 +448,6 @@ class FOFDataManager(metaclass=Singleton):
                 except KeyError:
                     pass
 
-                try:
-                    cd_interest_transfer = self._manually.at[date, 'cd_interest_transfer']
-                    if not pd.isnull(cd_interest_transfer):
-                        cd_interest_transfer = round(cd_interest_transfer, 2)
-                        # 扣掉累计应收银行存款利息
-                        total_cash += cd_interest_transfer
-                except KeyError:
-                    pass
-
                 try:
                     other_fees = self._manually.at[date, 'other_fees']
                     if not pd.isnull(other_fees):
@@ -431,6 +461,15 @@ class FOFDataManager(metaclass=Singleton):
                 # 应收银行存款利息
                 deposit_interest = round(cash_list.iat[-1] * self._DEPOSIT_INTEREST_PER_YEAR / self._DAYS_PER_YEAR_FOR_INTEREST, 2)
 
+                try:
+                    cd_interest_transfer = self._manually.at[date, 'cd_interest_transfer']
+                    if not pd.isnull(cd_interest_transfer):
+                        cd_interest_transfer = round(cd_interest_transfer, 2)
+                        # 计提费用相应扣掉
+                        deposit_interest += cd_interest_transfer
+                except KeyError:
+                    pass
+
                 # 记录一些信息
                 # TODO: 这里也需要在费用划拨时进行相应的扣除
                 if deposit_interest_list.empty:
@@ -460,6 +499,22 @@ class FOFDataManager(metaclass=Singleton):
                 today_net_assets = today_fund_mv + today_hedge_mv + misc_amount
                 net_assets_list.loc[date] = today_net_assets
 
+                try:
+                    fund_pos_info = pd.concat([self._fund_pos.loc[date, :].rename('share'), fund_nav.loc[date, :].rename('nav')], axis=1)
+                    fund_pos_info = fund_pos_info[fund_pos_info.share.notna()]
+                    fund_pos_info['asset_type'] = HoldingAssetType.MUTUAL
+                except KeyError:
+                    fund_pos_info = None
+                try:
+                    hedge_pos_info = pd.concat([self._hedge_pos.loc[date, :].rename('share'), self._hedge_nav.loc[date, :].rename('nav')], axis=1)
+                    hedge_pos_info = hedge_pos_info[hedge_pos_info.share.notna()]
+                    hedge_pos_info['asset_type'] = HoldingAssetType.HEDGE
+                except KeyError:
+                    hedge_pos_info = None
+                if fund_pos_info is not None or hedge_pos_info is not None:
+                    position = pd.concat([fund_pos_info, hedge_pos_info], axis=0).rename_axis(index='fund_id').reset_index()
+                    positions_list.loc[date] = json.dumps(position.to_dict(orient='records'))
+
                 # 计算修正净资产
                 try:
                     errors_to_be_fixed = self._manually.loc[self._manually.index <= date, ['admin_service_fee_error', 'custodian_fee_error', 'management_fee_error']].round(2).sum(axis=1).sum()
@@ -483,26 +538,49 @@ class FOFDataManager(metaclass=Singleton):
             self._fof_nav = fof_nav_list.rename_axis('datetime').to_frame(name='nav').reset_index()
             self._fof_nav['fof_id'] = FOFDataManager._FOF_ID
 
-    def dump_fof_nav_to_db(self):
+            self._fof_position = positions_list.rename_axis('datetime').to_frame(name='position').reset_index()
+            self._fof_position['fof_id'] = FOFDataManager._FOF_ID
+
+    def dump_fof_nav_and_pos_to_db(self):
         if self._fof_nav is not None:
             now_df = DerivedDataApi().get_fof_nav([FOFDataManager._FOF_ID])
             if now_df is not None:
-                now_df = now_df.drop(columns=['_update_time'])
+                now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted'])
                 # merge on all columns
                 df = self._fof_nav.merge(now_df, how='left', indicator=True, validate='one_to_one')
                 df = df[df._merge == 'left_only'].drop(columns=['_merge'])
             else:
                 df = self._fof_nav
-            for date in df.datetime.unique():
-                DerivedDataApi().delete_fof_nav(date_to_delete=date, fund_list=df[df.datetime == date].fof_id.to_list())
-            df.to_sql(FOFNav.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
-            print('[dump_fof_nav_to_db] done')
+            print(df)
+            if not df.empty:
+                for date in df.datetime.unique():
+                    DerivedDataApi().delete_fof_nav(date_to_delete=date, fof_id_list=df[df.datetime == date].fof_id.to_list())
+                df.to_sql(FOFNav.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
+                self._wechat_bot.send_fof_nav_update(df)
+            print('[dump_fof_nav_and_pos_to_db] dump nav done')
         else:
-            print('[dump_fof_nav_to_db] no nav, should calc it first')
+            print('[dump_fof_nav_and_pos_to_db] no nav, should calc it first')
+
+        if self._fof_position is not None:
+            now_df = DerivedDataApi().get_fof_position([FOFDataManager._FOF_ID])
+            if now_df is not None:
+                now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted'])
+                # merge on all columns
+                df = self._fof_position.merge(now_df, how='left', indicator=True, validate='one_to_one')
+                df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+            else:
+                df = self._fof_position
+            print(df)
+            if not df.empty:
+                for date in df.datetime.unique():
+                    DerivedDataApi().delete_fof_position(date_to_delete=date, fof_id_list=df[df.datetime == date].fof_id.to_list())
+                df.to_sql(FOFPosition.__table__.name, DerivedDatabaseConnector().get_engine(), index=False, if_exists='append')
+            print('[dump_fof_nav_and_pos_to_db] dump position done')
+        else:
+            print('[dump_fof_nav_and_pos_to_db] no position, should calc it first')
 
     def service_start(self):
         import os
-        from ...util.mail_retriever import MailAttachmentRetriever, IMAP_SPType
         from ..nav_reader.hedge_fund_nav_reader import HedgeFundNAVReader
 
         try:
@@ -511,17 +589,15 @@ class FOFDataManager(metaclass=Singleton):
             password = os.environ['EMAIL_PASSWORD']
         except KeyError as e:
             import sys
-            sys.exit(f'can not found enough params in env (e){e}')
+            sys.exit(f'[service_start] can not found enough params in env (e){e}')
 
-        mar = MailAttachmentRetriever(email_data_dir)
-        new_nav = mar.get_excels(IMAP_SPType.IMAP_QQ, user_name, password)
-        if new_nav is not None and new_nav:
-            print(new_nav)
-            hf_nav_r = HedgeFundNAVReader(email_data_dir, user_name, password)
-            hf_nav_r.read_navs_and_dump_to_db()
-
-        self.calc_nav()
-        # self.dump_fof_nav_to_db()
+        hf_nav_r = HedgeFundNAVReader(email_data_dir, user_name, password)
+        hf_nav_r.read_navs_and_dump_to_db()
+        try:
+            self.calc_nav()
+            self.dump_fof_nav_and_pos_to_db()
+        except Exception as e:
+            self._wechat_bot.send_fof_nav_update_failed(self._FOF_ID, f'calc fof nav failed (e){e}')
 
     @staticmethod
     def _concat_assets_price(main_asset: pd.DataFrame, secondary_asset: pd.Series) -> pd.DataFrame:
@@ -535,7 +611,10 @@ class FOFDataManager(metaclass=Singleton):
     # 以下是一些获取数据的接口
     @staticmethod
     def get_fof_info(fof_id: Tuple[str] = ()):
-        return BasicDataApi().get_fof_info(fof_id)
+        fof_info = BasicDataApi().get_fof_info(fof_id)
+        if fof_id is None:
+            return
+        return fof_info.sort_values(by=['fof_id', 'datetime']).iloc[-1]
 
     @staticmethod
     def get_fof_asset_allocation(fof_id: Tuple[str] = ()):
@@ -550,7 +629,7 @@ class FOFDataManager(metaclass=Singleton):
         fof_nav = DerivedDataApi().get_fof_nav([fof_id])
         if fof_nav is None or fof_nav.empty:
             return
-        fof_nav = fof_nav.drop(columns=['_update_time', 'fof_id'])
+        fof_nav = fof_nav.drop(columns=['update_time', 'fof_id', 'create_time', 'is_deleted'])
         if ref_index_id:
             index_price = BasicDataApi().get_index_price(index_list=[ref_index_id])
             if index_price is None or index_price.empty:
@@ -573,8 +652,211 @@ class FOFDataManager(metaclass=Singleton):
     @staticmethod
     def get_hedge_fund_nav(fund_id: Tuple[str] = ()):
         df = BasicDataApi().get_hedge_fund_nav(fund_id)
+        if df is None:
+            return
         return df.sort_values(by=['fund_id', 'datetime', 'insert_time']).drop_duplicates(subset=['fund_id', 'datetime'], keep='last')
 
+    @staticmethod
+    def get_investor_return(fof_id: str, *, investor_id_list: Tuple[str] = (), start_date: Optional[datetime.date] = None, end_date: Optional[datetime.date] = None) -> Optional[pd.DataFrame]:
+        '''计算投资者收益'''
+
+        fof_nav = FOFDataManager().get_fof_nav(fof_id)
+        if fof_nav is None:
+            return
+        fof_scale_info = FOFDataManager().get_fof_scale_alteration([fof_id])
+        if fof_scale_info is None:
+            return
+
+        fof_nav = fof_nav.set_index('datetime')
+        if start_date:
+            fof_nav = fof_nav[fof_nav.index >= start_date]
+        if end_date:
+            fof_nav = fof_nav[fof_nav.index <= end_date]
+        investor_mvs = defaultdict(list)
+        for row in fof_scale_info.itertuples(index=False):
+            if investor_id_list and row.investor_id not in investor_id_list:
+                continue
+            if start_date is not None:
+                actual_start_date = max(row.confirmed_date, start_date)
+            else:
+                actual_start_date = row.confirmed_date
+            try:
+                init_mv = fof_nav.at[actual_start_date, 'nav'] * row.unit_total
+            except KeyError:
+                init_mv = row.unit_total
+            try:
+                init_water_line = fof_nav.at[row.confirmed_date, 'nav']
+            except KeyError:
+                init_water_line = 1
+            investor_mvs[row.investor_id].append({'start_date': actual_start_date, 'amount': row.amount, 'share': row.unit_total, 'init_mv': init_mv, 'init_water_line': init_water_line})
+
+        investor_returns = {}
+        latest_nav = fof_nav.nav.iat[-1]
+        for investor_id, datas in investor_mvs.items():
+            datas = pd.DataFrame(datas)
+            # TODO: hard code 0.2 and 4 and the second param should be acc nav
+            v_nav = FOFDataManager._do_calc_v_net_value(latest_nav, latest_nav, datas.init_water_line, 0.2, 4)
+            # 这里确保单取的几个Series的顺序不会发生任何变化 这样直接运算才是OK的
+            latest_mv = (v_nav * datas.share).sum()
+            total_share = datas.share.sum()
+            avg_v_nav = latest_mv / total_share
+            total_rr = latest_mv / datas.init_mv.sum() - 1
+            investor_returns[investor_id] = {'v_nav': avg_v_nav, 'total_rr': total_rr, 'shares': total_share}
+            print(json.dumps([{'fof_id': FOFDataManager._FOF_ID, 'v_nav': avg_v_nav, 'amount': datas.amount.sum(), 'shares': total_share}]))
+        return pd.DataFrame.from_dict(investor_returns, orient='index')
+
+    @staticmethod
+    def calc_share_by_subscription_amount(fof_id: str, amount: float, confirmed_date: datetime.date) -> Optional[float]:
+        '''根据金额和日期计算申购fof产品的确认份额'''
+
+        fof_info: Optional[pd.DataFrame] = FOFDataManager.get_fof_info([fof_id])
+        if fof_info is None:
+            return
+        fof_nav = FOFDataManager().get_fof_nav(fof_id)
+        if fof_nav is None:
+            return
+        try:
+            # 用申购金额除以确认日的净值以得到份额
+            return amount / (1 + fof_info.subscription_fee) / fof_nav.loc[fof_nav.datetime == confirmed_date, 'nav'].array[-1]
+        except KeyError:
+            return
+
+    @staticmethod
+    def remove_hedge_nav_data(fund_id: str, start_date: str = '', end_date: str = ''):
+        BasicDataApi().delete_hedge_fund_nav(fund_id_to_delete=fund_id, start_date=start_date, end_date=end_date)
+
+    @staticmethod
+    def upload_hedge_nav_data(datas: bytes, file_name: str, hedge_fund_id: str) -> bool:
+        '''上传单个私募基金净值数据'''
+        '''目前支持：私募排排网 朝阳永续'''
+
+        if not datas:
+            return True
+
+        try:
+            # 下边这个分支是从私募排排网上多只私募基金净值数据的文件中读取数据
+            # if file_type == 'csv':
+            #     hedge_fund_info = BasicDataApi().get_hedge_fund_info()
+            #     if hedge_fund_info is None:
+            #         return
+            #     fund_name_map = hedge_fund_info.set_index('brief_name')['fund_id'].to_dict()
+            #     fund_name_map['日期'] = 'datetime'
+            #     df = pd.read_csv(io.BytesIO(datas), encoding='gbk')
+            #     lacked_map = set(df.columns.array) - set(fund_name_map.keys())
+            #     assert not lacked_map, f'lacked hedge fund name map {lacked_map}'
+            #     df = df.rename(columns=fund_name_map).set_index('datetime').rename_axis(columns='fund_id')
+            #     df = df.stack().to_frame('adjusted_net_value').reset_index()
+            #     # validate = 'many_to_many'
+            try:
+                if '产品详情-历史净值' in file_name:
+                    df = pd.read_excel(io.BytesIO(datas), usecols=['净值日期', '单位净值', '累计净值', '复权累计净值'], na_values='--')
+                    df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+                    df['fund_id'] = hedge_fund_id
+                elif '业绩走势' in file_name:
+                    df = pd.read_excel(io.BytesIO(datas), usecols=['净值日期', '净值(分红再投)'], na_values='--')
+                    df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+                    df['fund_id'] = hedge_fund_id
+                else:
+                    assert False
+            except Exception:
+                try:
+                    try:
+                        df = pd.read_excel(io.BytesIO(datas), na_values='--')
+                    except Exception:
+                        df = pd.read_csv(io.BytesIO(datas), na_values='--')
+                except Exception:
+                    print(f'[upload_hedge_nav_data] can not read data from this file (file name){file_name} (fund_id){hedge_fund_id}')
+                    return False
+                else:
+                    df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+                    df['fund_id'] = hedge_fund_id
+
+            df = df[df.drop(columns=['fund_id', 'datetime']).notna().any(axis=1)]
+            now_df = BasicDataApi().get_hedge_fund_nav([hedge_fund_id])
+            if now_df is not None:
+                now_df = now_df.drop(columns=['update_time', 'create_time', 'insert_time', 'is_deleted', 'calc_date']).astype({'net_asset_value': 'float64', 'acc_unit_value': 'float64', 'v_net_value': 'float64', 'adjusted_net_value': 'float64'})
+                df = df.reindex(columns=now_df.columns).astype(now_df.dtypes.to_dict())
+                df['datetime'] = pd.to_datetime(df.datetime, infer_datetime_format=True).dt.date
+                # merge on all columns
+                df = df.round(6).merge(now_df, how='left', indicator=True, validate='one_to_one')
+                df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+            if not df.empty:
+                df['insert_time'] = datetime.datetime.now()
+                print(f'[upload_hedge_nav_data] try to insert data to db (df){df}')
+                df.to_sql(HedgeFundNAV.__table__.name, BasicDatabaseConnector().get_engine(), index=False, if_exists='append')
+            else:
+                print(f'[upload_hedge_nav_data] empty df, nothing to do')
+            return True
+        except Exception as e:
+            print(f'[upload_hedge_nav_data] failed, got exception {e}')
+            return False
+
+    @staticmethod
+    def virtual_backtest(hedge_fund_ids: Tuple[str], benchmark_ids: Tuple[str] = ()) -> Tuple[pd.DataFrame, List[str]]:
+        '''
+        对私募基金做虚拟回测
+
+        Parameters
+        ----------
+        hedge_fund_ids : Tuple[str]
+            私募基金ID列表
+        benchmark_ids : Tuple[str]
+            指数ID列表
+
+        Returns
+        -------
+        Tuple[pd.DataFrame, List[str]]
+        fof nav df and hedge fund ids that lacked
+        '''
+
+        DEFAULT_INCENTIVE_RATIO = 0.2
+        INITIAL_CASH = 10000000
+        INIT_NAV = 1
+        UNIT_TOTAL = INITIAL_CASH / INIT_NAV
+
+        fund_info = BasicDataApi().get_hedge_fund_info(hedge_fund_id)
+        fund_info = fund_info.set_index('fund_id')
+
+        fund_nav = BasicDataApi().get_hedge_fund_nav(hedge_fund_id)
+        adj_nav = fund_nav.pivot(index='datetime', columns='fund_id', values='adjusted_net_value')
+        fund_nav = FOFDataManager._do_calc_v_net_value(adj_nav, adj_nav, fund_info.water_line, fund_info.incentive_fee_ratio.fillna(DEFAULT_INCENTIVE_RATIO), fund_info.v_nav_decimals)
+        # FIXME 这里暂时不能直接用数据库里存的虚拟净值 因为这个是邮件里发过来的单位净值虚拟后净值 而上边算出来的是复权净值虚拟后净值
+        # fund_nav = fund_nav.pivot(index='datetime', columns='fund_id', values='v_net_value')
+        # fund_nav = fund_nav.combine_first(v_nav)
+
+        # 计算虚拟净值
+        trading_day = BasicDataApi().get_trading_day_list(start_date=fund_nav.index.array[0], end_date=fund_nav.index.array[-1])
+        fund_nav_ffilled = fund_nav.reindex(trading_day.datetime).ffill()
+        fund_nav_ffilled = fund_nav_ffilled.loc[:, fund_nav_ffilled.notna().any(axis=0)].loc[fund_nav_ffilled.notna().any(axis=1), :]
+        # raw_wgts = [10, 15, 10, 10, 10, 10, 15, 20]
+        # wgts = pd.Series(raw_wgts, index=name_dict.values())
+        positions = []
+        mvs = []
+        navs = []
+        # 初始市值
+        mv = INITIAL_CASH
+        for index, s in fund_nav_ffilled.iterrows():
+            s = s[s.notna()]
+            if positions:
+                # 最新市值
+                mv = (positions[-1][1] * s).sum()
+
+            if not positions or (s.size != positions[-1][1].size):
+                # 调仓
+                # new_wgts = wgts.loc[s.index]
+                # 各标的目标权重
+                # new_wgts /= new_wgts.sum()
+                # 新的各标的持仓份数
+                # shares = (mv * new_wgts) / s
+                shares = (mv / s.size) / s
+                positions.append((index, shares))
+
+            nav = mv / UNIT_TOTAL
+            mvs.append(mv)
+            navs.append(nav)
+        fof_nav = pd.Series(navs, index=fund_nav_ffilled.index, name='fof_nav')
+        return fof_nav
+
     # TODO: 提供接口支持对数据的修改
 
 
