@@ -20,7 +20,7 @@ from .etl_tool import S3RetrieverForDM
 from ..view.raw_models import CSIndexComponent
 from ..view.basic_models import *
 from ..view.derived_models import *
-from .combine_single_fac import combine_factor
+from .combine_single_fac import combine_factor, combine_factor_test
 
 INDEX_REPLACE = {'active':'csi_stockfund',
                  'conv_bond':'csi_convex'}
@@ -273,7 +273,9 @@ class FundDataManager(DataManager):
             _tm_basic_filter_info = time.time()
             self.dts.fund_list = set(_filter_fund_info.fund_id)
             self.dts.all_fund_list = self.dts.fund_list.union(self.dts.active_fund_info.fund_id)
-            in_market_fund_list = self.dts.fund_status_latest[~self.dts.fund_status_latest['trade_status'].isnull()].fund_id.tolist()
+            _in_market_fund_df = self.dts.fund_status_latest[~self.dts.fund_status_latest['trade_status'].isnull()]
+            in_market_fund_list = _in_market_fund_df[~((_in_market_fund_df.redem_status == '正常赎回') & (_in_market_fund_df.purch_status == '正常申购'))]
+            in_market_fund_list = in_market_fund_list.fund_id.tolist()
             close_fund_list = _fund_open_info.fund_id.tolist()
             self.dts.fund_ipo_stats = fetch_table(quant_session, FundIPOStats)
             self.dts.fund_conv_stats = fetch_table(quant_session, FundConvStats)
@@ -428,8 +430,9 @@ class FundDataManager(DataManager):
                 df.loc[:, 'index_id'] = index_id
                 res.append(df)
             _index_pct_df = pd.concat(res, axis=0)
-            self.dts.index_pct = _index_pct_df[['datetime','index_id','pe_pct_5_3','pb_pct_5_3','ps_pct_5_3']].rename(
-                columns={'pe_pct_5_3':'pe_pct','pb_pct_5_3':'pb_pct','ps_pct_5_3':'ps_pct'}).pivot_table(index=['datetime', 'index_id'], values=['pe_pct', 'pb_pct', 'ps_pct'])
+            index_val_list = list(TaaTunerParam.INDEX_PCT_SELECTED.keys())
+            self.dts.index_pct = _index_pct_df[['datetime','index_id'] + index_val_list].rename(
+                columns=TaaTunerParam.INDEX_PCT_SELECTED).pivot_table(index=['datetime', 'index_id'], values=['pe_pct', 'pb_pct', 'ps_pct'])
             self.dts.index_date_list = list(self.dts.index_pct.index.remove_unused_levels().levels[0])
             _tm_deriv_apply_index_pct = time.time()
 
@@ -498,7 +501,9 @@ class FundDataManager(DataManager):
                                             how='left', left_on=['fund_id','datetime'], right_on=['fund_id','datetime'])
             self.dts.fund_indicator['fund_size'] = self.dts.fund_indicator['fund_size'].fillna(0)
             _single_fac_df = combine_factor()
+            #_single_fac_df = combine_factor_test()
             self.dts.fund_indicator = pd.merge(self.dts.fund_indicator, _single_fac_df, how='left', on=['datetime','fund_id'])
+            
             _tm_deriv_apply_manager_rank = time.time()
 
             if use_weekly_monthly_indicators:
@@ -622,6 +627,8 @@ class FundDataManager(DataManager):
                                     & (_fund_indicator_whole.datetime < next_day)].copy()
                 _df['index_id'] = _index_id
                 res.append(_df)
+        if not res:
+            return
         return pd.concat(res)
 
     def _get_annual_alpha_beta(self, _fund_indicator_annual_fund_i, year):
