@@ -1,10 +1,11 @@
 
+
+from typing import Optional, List
 import re
 import datetime
 import os
 import math
 
-import numpy as np
 import pandas as pd
 
 from ...util.mail_retriever import MailAttachmentRetriever, IMAP_SPType, UID_FILE_NAME
@@ -16,7 +17,7 @@ from ..api.basic import BasicDataApi
 
 class HedgeFundNAVReader:
 
-    _COLUMNS_DICT = {
+    COLUMNS_DICT = {
         '基金代码': 'fund_id',
         '产品代码': 'fund_id',
         '基金名称': 'fund_name',
@@ -26,9 +27,13 @@ class HedgeFundNAVReader:
         '计提前单位净值': 'net_asset_value',
         '基金份额累计净值': 'acc_unit_value',
         '累计单位净值': 'acc_unit_value',
+        '累计净值': 'acc_unit_value',
         '虚拟后净值': 'v_net_value',
         '虚拟净值': 'v_net_value',
         '计提后单位净值': 'v_net_value',
+        '复权累计净值': 'adjusted_net_value',
+        '复权净值': 'adjusted_net_value',
+        '净值(分红再投)': 'adjusted_net_value',
         '日期': 'datetime',
         '净值日期': 'datetime',
         '业务日期': 'datetime',
@@ -44,53 +49,78 @@ class HedgeFundNAVReader:
         self._wechat_bot = WechatBot()
 
     @staticmethod
-    def _read_for_HuangChengZhiYuanNo3_from_cms(file_path: str) -> pd.DataFrame:
+    def _read_for_HuangChengZhiYuanNo3WithoutV(file_path: str) -> pd.DataFrame:
         df = pd.read_excel(file_path, header=None)
         df.iloc[:, 0] = df.iloc[:, 0].map(lambda x: x.split('：')[0])
         df = df.set_index(df.columns[0])
         pat = r'\d+'
         date = re.findall(pat, df.index.array[2])
         df = df.T.loc[:, ['基金代码', '基金名称', '基金份额净值', '基金份额累计净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT).assign(datetime=pd.to_datetime('-'.join(date), infer_datetime_format=True).date())
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT).assign(datetime=pd.to_datetime('-'.join(date), infer_datetime_format=True).date())
         return df
 
     @staticmethod
     def _read_for_HuangChengZhiYuanNo3(file_path: str) -> pd.DataFrame:
         df = pd.read_excel(file_path)
         df = df.loc[:, ['产品代码', '产品名称', '业务日期', '单位净值', '累计单位净值', '虚拟后净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT)
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
         df['datetime'] = pd.to_datetime(df.datetime.astype(str), infer_datetime_format=True).dt.date
         return df
 
     @staticmethod
-    def _read_for_TianYanGuangQuan_from_tg(file_path: str) -> pd.DataFrame:
-        df = pd.read_excel(file_path, skipfooter=1)
+    def _read_for_TianYanGuangQuanWithoutV(file_path: str) -> pd.DataFrame:
+        df = pd.read_excel(file_path)
         df = df.loc[:, ['产品名称', '产品代码', '净值日期', '单位净值', '累计单位净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT)
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
         return df
 
     @staticmethod
     def _read_for_TianYanGuangQuan(file_path: str) -> pd.DataFrame:
         df = pd.read_excel(file_path, skipfooter=1)
         df = df.loc[:, ['基金名称', '基金代码', '净值日期', '计算日期', '单位净值', '累计单位净值', '虚拟净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT)
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
         return df
 
     @staticmethod
     def _read_for_AnXianHuaMuChangSheng(file_path: str) -> pd.DataFrame:
         df = pd.read_excel(file_path)
         df = df.loc[:, ['产品名称', '产品代码', '日期', '单位净值', '累计单位净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT)
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
         return df
 
     @staticmethod
     def _read_for_WuZhiCTA1(file_path: str) -> pd.DataFrame:
         df = pd.read_excel(file_path)
         df = df.loc[:, ['产品名称', '产品代码', '净值日期', '计提前单位净值', '累计单位净值', '计提后单位净值']]
-        df = df.rename(columns=HedgeFundNAVReader._COLUMNS_DICT)
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
         return df
 
-    def _notify_error_event(err_msg: str):
+    @staticmethod
+    def _read_for_WuZhiCTA1WithoutV(file_path: str) -> pd.DataFrame:
+        df = pd.read_excel(file_path)
+        df = df.loc[:, ['产品名称', '产品代码', '日期', '单位净值', '累计单位净值']]
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+        return df
+
+    @staticmethod
+    def _read_for_XinPu10(file_path: str) -> pd.DataFrame:
+        df = pd.read_excel(file_path)
+        df = df.loc[:, ['产品名称', '产品代码', '业务日期', '单位净值', '累计单位净值', '虚拟后净值']]
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+        return df
+
+    @staticmethod
+    def _read_for_TaiChuangXiShuoCTA(file_path: str) -> pd.DataFrame:
+        df = pd.read_excel(file_path, index_col=0, skiprows=3, usecols=[0, 1])
+        dates_data = df.index.name.split('：')
+        assert (len(dates_data) >= 2) and (dates_data[0] == '日期'), 'invalid format from TaiChuangXiShuoCTA'
+        df = df.loc[['单位净值', '累计单位净值'], :].T
+        df['datetime'] = pd.to_datetime(dates_data[1], infer_datetime_format=True).date()
+        df['fund_id'] = 'SNH765'
+        df = df.rename(columns=HedgeFundNAVReader.COLUMNS_DICT)
+        return df
+
+    def _notify_error_event(self, err_msg: str):
         print(f'[read_navs_and_dump_to_db] {err_msg}')
         self._wechat_bot.send_hedge_fund_nav_update_failed(err_msg)
 
@@ -109,7 +139,7 @@ class HedgeFundNAVReader:
             data = mar.get_excels(IMAP_SPType.IMAP_QQ, self._user_name, self._password, uid_last)
         except Exception as e:
             self._notify_error_event(f'FATAL ERROR!! get new data of hedge fund nav failed (e){e}')
-            return
+            return False
 
         uid_last_succeed: Optional[bytes] = None
         df_list: List[pd.DataFrame] = []
@@ -121,17 +151,23 @@ class HedgeFundNAVReader:
 
             try:
                 if 'SGM473' in name:
-                    df = HedgeFundNAVReader._read_for_HuangChengZhiYuanNo3_from_cms(file_path)
+                    df = HedgeFundNAVReader._read_for_HuangChengZhiYuanNo3WithoutV(file_path)
                 elif '华澄致远三号' in name:
                     df = HedgeFundNAVReader._read_for_HuangChengZhiYuanNo3(file_path)
                 elif 'SLC213_天演广全' in name:
-                    df = HedgeFundNAVReader._read_for_TianYanGuangQuan_from_tg(file_path)
+                    df = HedgeFundNAVReader._read_for_TianYanGuangQuanWithoutV(file_path)
                 elif '天演广全' in name:
                     df = HedgeFundNAVReader._read_for_TianYanGuangQuan(file_path)
                 elif '安贤花木长盛' in name:
                     df = HedgeFundNAVReader._read_for_AnXianHuaMuChangSheng(file_path)
                 elif '吾执CTA一号' in name:
                     df = HedgeFundNAVReader._read_for_WuZhiCTA1(file_path)
+                elif '产品净值情况' in name:
+                    df = HedgeFundNAVReader._read_for_WuZhiCTA1WithoutV(file_path)
+                elif '信朴10号私募基金' in name:
+                    df = HedgeFundNAVReader._read_for_XinPu10(file_path)
+                elif 'SNH765泰创-羲朔CTA' in name:
+                    df = HedgeFundNAVReader._read_for_TaiChuangXiShuoCTA(file_path)
                 else:
                     raise NotImplementedError('unknown hedge fund nav file from attachment')
             except Exception as e:
@@ -139,17 +175,13 @@ class HedgeFundNAVReader:
                 continue
 
             try:
-                # FIXME 暂时不需要处理没有虚拟净值的情况
-                if 'v_net_value' in df.columns:
-                    df = df.drop(columns=['fund_name'])
-                    df['insert_time'] = datetime.datetime.now()
-                    df = self._dump_to_db(df)
-                    if df is not None:
-                        df_list.append(df)
-                    else:
-                        print(f'[read_navs_and_dump_to_db] duplicated data, do not process it (name){name}')
+                df = df.drop(columns=['fund_name'], errors='ignore')
+                df['insert_time'] = datetime.datetime.now()
+                df = self._dump_to_db(df)
+                if df is not None:
+                    df_list.append(df)
                 else:
-                    print(f'[read_navs_and_dump_to_db] no virtual net value, do not process it (name){name}')
+                    print(f'[read_navs_and_dump_to_db] duplicated data, do not process it (name){name}')
                 # 走到这里都认为是已经处理完了这条数据
                 uid_last_succeed = uid
             except Exception as e:
@@ -162,7 +194,7 @@ class HedgeFundNAVReader:
                 print(whole_df)
             except Exception as e:
                 self._notify_error_event(f'{e} (concat)')
-                return
+                return False
             else:
                 self._wechat_bot.send_hedge_fund_nav_update(whole_df)
                 print(f'[read_navs_and_dump_to_db] done (uid_last){uid_last_succeed} (df){whole_df}')
@@ -172,6 +204,8 @@ class HedgeFundNAVReader:
         if uid_last_succeed is not None:
             with open(os.path.join(self._read_dir, UID_FILE_NAME), 'wb') as f:
                 f.write(uid_last_succeed)
+                return True
+        return False
 
     def _dump_to_db(self, df: pd.DataFrame):
         def _check_after_merged(x: pd.Series, now_df: pd.DataFrame):
@@ -187,13 +221,14 @@ class HedgeFundNAVReader:
                 return x
 
         assert df.datetime.nunique() == 1, 'should have single datetime'
-        now_df = BasicDataApi().get_hedge_fund_nav(fund_id_list=df.fund_id.to_list())
+        now_df = BasicDataApi().get_hedge_fund_nav(fund_id_list=list(df.fund_id.unique()))
         if now_df is not None and not now_df.empty:
             # 同产品同日期的净值如果已经存在了且没有变化，就不写DB了
-            now_df = now_df.drop(columns=['_update_time']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset=['fund_id', 'datetime'], keep='last')
+            now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset=['fund_id', 'datetime'], keep='last')
+            now_df = now_df.astype({'adjusted_net_value': 'float64', 'v_net_value': 'float64'})
             df = df.reindex(columns=now_df.columns).astype(now_df.dtypes.to_dict())
             df = df.merge(now_df, how='left', on=['fund_id', 'datetime', 'net_asset_value', 'acc_unit_value', 'v_net_value'], indicator=True, validate='one_to_one')
-            df = df[df._merge == 'left_only'].drop(columns=['_merge', 'insert_time_y', 'calc_date_y']).rename(columns={'insert_time_x': 'insert_time', 'calc_date_x': 'calc_date'})
+            df = df[df._merge == 'left_only'].drop(columns=['_merge', 'insert_time_y', 'calc_date_y', 'adjusted_net_value_y']).rename(columns={'insert_time_x': 'insert_time', 'calc_date_x': 'calc_date', 'adjusted_net_value_x': 'adjusted_net_value'})
             if df.empty:
                 return
             # FIXME 没想到特别好的方法 遍历每一行再check一下
