@@ -449,7 +449,7 @@ class FundPosition:
             assert trade.is_buy, "fund volume cannot be negative"
             pos_item = self.add(trade.fund_id, trade.index_id, 0, 0, trade.trade_date or trade.submit_date, trade.is_permitted_fund)
 
-        p = trade.trade_date if trade.trade_price else trade.mark_price
+        p = trade.trade_price if trade.trade_price else trade.mark_price
         amt = trade.amount if trade.amount else trade.volume * p
         cash_left = self.cash + (-amt if trade.is_buy else amt)
         if cash_left <= -1e-6:
@@ -544,6 +544,12 @@ class TaaTunerParam:
         'sp500rmb':'pe_ttm',
     }
 
+    INDEX_PCT_SELECTED = {
+        'pe_pct_5_3':'pe_pct',
+        'pb_pct_5_3':'pb_pct',
+        'ps_pct_5_3':'ps_pct'
+    }
+    
 # dataclass默认有__eq__
 @dataclasses.dataclass
 class ScoreFilter:
