@@ -6,14 +6,25 @@ from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, SMALLINT, DATETIME,
 from ...constant import SectorType, IndexPriceSource, HoldingAssetType, FundStatus
 
 
-class Base():
+class Base:
     _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
 
 
+class FOFBase:
+    create_time = Column(DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP'))  # 创建时间
+    update_time = Column(DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
+    is_deleted = Column(BOOLEAN, nullable=False, server_default=text('FALSE'))  # 是否删除 默认不删除
+
+
 # make this column at the end of every derived table
 Base._update_time._creation_order = 9999
 Base = declarative_base(cls=Base)
 
+FOFBase.create_time._creation_order = 9997
+FOFBase.update_time._creation_order = 9998
+FOFBase.is_deleted._creation_order = 9999
+FOFBase = declarative_base(cls=FOFBase)
+
 
 class SectorInfo(Base):
     '''板块信息表'''
@@ -736,6 +747,7 @@ class OverseaFundNav(Base):
     codes = Column(CHAR(25), primary_key=True)  # 基金id
     datetime = Column(DATE, primary_key=True) # 日期
     nav = Column(DOUBLE(asdecimal=False)) # 净值
+
 class ETFInfo(Base):
 
     __tablename__ = 'etf_info'
@@ -768,7 +780,7 @@ class FundTag(Base):
     tag_name = Column(CHAR(64)) # 标签名
 
 
-class HedgeFundNAV(Base):
+class HedgeFundNAV(FOFBase):
     '''私募基金净值'''
 
     __tablename__ = 'hedge_fund_nav'
@@ -777,27 +789,32 @@ class HedgeFundNAV(Base):
     datetime = Column(DATE, primary_key=True)  # 日期
     insert_time = Column(DATETIME, primary_key=True)  # 插入时间
     calc_date = Column(DATE)  # 计算日期
-    net_asset_value = Column(DOUBLE(asdecimal=False), nullable=False)  # 单位净值
+    net_asset_value = Column(DOUBLE(asdecimal=False))  # 单位净值
     acc_unit_value = Column(DOUBLE(asdecimal=False))  # 累计净值
-    v_net_value = Column(DOUBLE(asdecimal=False))  # 虚拟净值
+    adjusted_net_value = Column(DOUBLE(asdecimal=False)) # 复权净值
+    v_net_value = Column(DOUBLE(asdecimal=False))  # 单位净值的虚拟后净值
 
-    __table_args__ = (
-        Index('idx_hedge_fund_nav_datetime', 'datetime'),
-    )
+    # 不能加这个 会影响fof backend项目
+    # __table_args__ = (
+    #     Index('idx_hedge_fund_nav_datetime', 'datetime'),
+    # )
 
 
-class HedgeFundInfo(Base):
+class HedgeFundInfo(FOFBase):
     '''私募基金信息'''
 
     __tablename__ = 'hedge_fund_info'
 
     fund_id = Column(CHAR(16), primary_key=True)  # 基金ID
     fund_name = Column(TEXT, nullable=False)  # 基金名称
+    brief_name = Column(TEXT, nullable=False)  # 基金简称
     manager_id = Column(CHAR(16), nullable=False)  # 私募基金公司ID
-    water_line = Column(DOUBLE(asdecimal=False), nullable=False)  # 水位线
-    incentive_fee_mode = Column(TEXT, nullable=False)  # 业绩报酬计提方法
-    incentive_fee_ratio = Column(DOUBLE(asdecimal=False), nullable=False)  # 业绩计提比例
+    water_line = Column(DOUBLE(asdecimal=False), nullable=False)  # 初始水位线
+    incentive_fee_mode = Column(TEXT)  # 业绩报酬计提方法
+    incentive_fee_ratio = Column(DOUBLE(asdecimal=False))  # 业绩计提比例
     v_nav_decimals = Column(SMALLINT, nullable=False)  # 虚拟净值精度
+    size = Column(DOUBLE(asdecimal=False))  # 基金规模
+    manager = Column(TEXT)  # 私募基金经理
 
 
 # class HedgeFundMgrInfo(Base):
@@ -809,7 +826,7 @@ class HedgeFundInfo(Base):
 #     manager_name = Column(TEXT, nullable=False)  # 私募基金公司名称
 
 
-class FOFInfo(Base):
+class FOFInfo(FOFBase):
     '''fof产品信息'''
 
     __tablename__ = 'fof_info'
@@ -828,26 +845,29 @@ class FOFInfo(Base):
     lock_up_period = Column(TEXT, nullable=False)  # 锁定期
     incentive_fee_mode = Column(TEXT, nullable=False)  # 业绩报酬计提方法
     incentive_fee = Column(TEXT, nullable=False)  # 业绩报酬提取比例
-    current_depoist_rate = Column(DOUBLE(asdecimal=False), nullable=False)  # 银行活期存款利率
+    current_deposit_rate = Column(DOUBLE(asdecimal=False), nullable=False)  # 银行活期存款利率
     initial_raised_fv = Column(DOUBLE(asdecimal=False), nullable=False)  # 初始募集面值
-    initial_net_value = Column(DOUBLE(asdecimal=False), nullable=False)  # 期初总净值
+    initial_net_value = Column(DOUBLE(asdecimal=False), nullable=False)  # 期初总资产净值
 
 
-class FOFScaleAlteration(Base):
+class FOFScaleAlteration(FOFBase):
     '''fof规模变动记录'''
 
     __tablename__ = 'fof_scale_alteration'
 
+    id = Column(Integer, primary_key=True, autoincrement=True)
     fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
     datetime = Column(DATE, primary_key=True)  # 日期
-    id = Column(Integer, primary_key=True, autoincrement=True)
+    investor_id = Column(CHAR(16), nullable=False)  # 投资人ID
     confirmed_date = Column(DATE)  # 确认日期
     deposited_date = Column(DATE)  # 入账日期
     amount = Column(DOUBLE(asdecimal=False))  # 被申购金额
     share = Column(DOUBLE(asdecimal=False))  # 被赎回份额
+    unit_total = Column(DOUBLE(asdecimal=False))  # 确认份额
+    asset_type = Column(Enum(HoldingAssetType), nullable=False)  # 资产类型
 
 
-class FOFAssetAllocation(Base):
+class FOFAssetAllocation(FOFBase):
     '''fof资产配置记录'''
 
     __tablename__ = 'fof_asset_allocation'
@@ -864,7 +884,7 @@ class FOFAssetAllocation(Base):
     unit_total = Column(DOUBLE(asdecimal=False))  # 确认份额
 
 
-class FOFManually(Base):
+class FOFManually(FOFBase):
     '''fof手工校正数据'''
 
     __tablename__ = 'fof_manually'
@@ -877,3 +897,25 @@ class FOFManually(Base):
     management_fee_error = Column(DOUBLE(asdecimal=False))  # 管理费误差
     custodian_fee_error = Column(DOUBLE(asdecimal=False))  # 托管费误差
     admin_service_fee_error = Column(DOUBLE(asdecimal=False))  # 行政服务费误差
+
+
+class FOFInvestorInfo(Base):
+    '''fof投资人信息'''
+
+    __tablename__ = 'fof_investor_info'
+
+    investor_id = Column(CHAR(16), primary_key=True)  # 客户合同号(投资人ID)
+    investor_name = Column(CHAR(8), nullable=False)  # 姓名
+
+
+class FOFInvestorPosition(FOFBase):
+    '''fof投资人持仓信息'''
+
+    __tablename__ = 'fof_investor_position'
+
+    investor_id = Column(CHAR(16), primary_key=True)  # 投资人ID
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    v_nav = Column(DOUBLE(asdecimal=False))  # 对应到投资人的虚拟净值
+    amount = Column(DOUBLE(asdecimal=False))  # 总投资额
+    shares = Column(DOUBLE(asdecimal=False))  # 总份额
