@@ -5,14 +5,25 @@ from sqlalchemy.dialects.mysql import DOUBLE, DATE, DATETIME, SMALLINT
 from ...constant import StockFactorType, IndClassType
 
 
-class Base():
+class Base:
     _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
 
 
+class FOFBase:
+    create_time = Column(DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP'))  # 创建时间
+    update_time = Column(DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
+    is_deleted = Column(BOOLEAN, nullable=False, server_default=text('FALSE'))  # 是否删除 默认不删除
+
+
 # make this column at the end of every derived table
 Base._update_time._creation_order = 9999
 Base = declarative_base(cls=Base)
 
+FOFBase.create_time._creation_order = 9997
+FOFBase.update_time._creation_order = 9998
+FOFBase.is_deleted._creation_order = 9999
+FOFBase = declarative_base(cls=FOFBase)
+
 
 class IndexVolatility(Base):
     '''指数波动率'''
@@ -685,7 +696,7 @@ class FundIndustryExposure(Base):
     value = Column(TEXT, nullable=False)  # JSON格式的行业暴露数据
 
 
-class FOFNav(Base):
+class FOFNav(FOFBase):
     '''fof净值数据'''
 
     __tablename__ = 'fof_nav'
@@ -693,3 +704,13 @@ class FOFNav(Base):
     fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
     datetime = Column(DATE, primary_key=True)  # 日期
     nav = Column(DOUBLE(asdecimal=False), nullable=False)  # 单位净值
+
+
+class FOFPosition(FOFBase):
+    '''fof持仓数据'''
+
+    __tablename__ = 'fof_position'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    position = Column(TEXT, nullable=False)  # 持仓信息
