@@ -157,7 +157,7 @@ class FundDailyCollection(Base):
     timing_score = Column(DOUBLE(asdecimal=False))  # 被动指数型、被动指数型债券(均为二级分类)基金和货基(一二级分类相同)为管理规模能力，其他为择时能力
     selection_score = Column(DOUBLE(asdecimal=False))  # 被动指数型、被动指数型债券(均为二级分类)基金和货基(一二级分类相同)为机构观点，其他为选证能力
     team_score = Column(DOUBLE(asdecimal=False))  # 基金公司团队能力
-    total_score = Column(DOUBLE(asdecimal=False), nullable=False)  # 总分
+    total_score = Column(DOUBLE(asdecimal=False))  # 总分
 
     # fund recommend part
     sector_list = Column(TEXT) # 板块列表 需要json.loads
