@@ -682,12 +682,12 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
 
     def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None, is_real_trade=False):
 
+        trade_list = []
         if dt in self._new_fund_position:
             position_list = self._new_fund_position[dt]
             cur_cash = sum([i.get('amount') for i in position_list])
             self.cur_fund_position = FundPosition(cash=cur_cash)
 
-            trade_list = []
             for i in position_list:
                 fund_id = i['fund_id']
                 _trade = FundTrade(
@@ -699,14 +699,12 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
                     submit_date=dt
                 )
                 trade_list.append(_trade)
-            return trade_list
 
         if dt in self._new_append_trades:
             if not self.cur_fund_position:
                 self.cur_fund_position = FundPosition(cash=0)
             append_trades = self._new_append_trades[dt]
 
-            trade_list = []
             for i in append_trades:
                 trade_type = i.get('trade_type')
                 fund_id = i['fund_id']
@@ -723,7 +721,7 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
                     self.cur_fund_position.cash = self.cur_fund_position.cash + i['amount']
                     trade_list.append(_trade)
                 elif trade_type == 'sell':
-                    if i['sell_rate']:
+                    if i.get('sell_rate'):
                         volume = self.cur_fund_position.funds.get(fund_id).volume * i['sell_rate']
                     else:
                         volume = i['volume']
@@ -736,8 +734,7 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
                         submit_date=dt
                     )
                     trade_list.append(_trade)
-            return trade_list
-        return []
+        return trade_list
 
     def update_cash(self, traded_list):
         for trd in traded_list:
@@ -747,6 +744,7 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
             else:
                 self.current_withdraw += trd.amount
                 self.withdraws += trd.amount
+                self.cur_fund_position.cash = self.cur_asset_position.cash - trd.amount
 
     def update_reporter(self, dt, trade_list, cur_asset_position: AssetPosition,
                         cur_fund_position: FundPosition = None, current_deposit=None, current_withdraw=None):
