@@ -240,18 +240,24 @@ class ReportHelper:
         return pos_df
 
     def get_last_target_fund_allocation(self, target_fund):
+        dt = list(self.fund_position_history.keys())[-1]
         res = []
         fund_info_dic = self.fund_info[['fund_id','desc_name']].set_index('fund_id').to_dict()['desc_name']
         for fund_id, fund_w in target_fund.items():
+            _score_dict = self.fund_score_dm[dt][fund_w.index_id][self.score_select[fund_w.index_id]]
+            _score_list = sorted(_score_dict.items(), key=lambda x:x[1], reverse=True)
+            _fund_list = [i[0] for i in _score_list]
+            rank_i = _fund_list.index(fund_w.fund_id) + 1 if fund_w.fund_id in _fund_list else 10000
             dic = {'fund_id':fund_w.fund_id,
                 'index_id':fund_w.index_id, 
                 'weight':round(fund_w.fund_wgt,3),
-                'desc_name':fund_info_dic[fund_w.fund_id]}
+                'desc_name':fund_info_dic[fund_w.fund_id],
+                'rank':rank_i}
             res.append(dic)
         target_fund = pd.DataFrame(res).set_index('fund_id')
         target_fund = target_fund.join(self.fund_info.set_index('fund_id')[['fund_manager']])
         target_fund.index.name = list(self.fund_position_history.keys())[-1]
-        return target_fund.sort_values('weight', ascending = False)[['index_id','weight','desc_name','fund_manager']]
+        return target_fund.sort_values('weight', ascending = False)[['index_id','weight','rank','desc_name','fund_manager']]
 
     def get_fund_trade(self):
         fsdf = self.fund_score.copy()
@@ -267,7 +273,10 @@ class ReportHelper:
             ft['desc_name'] = fund_info_df.loc[ft['fund_id'], 'desc_name']
             index_id = fund_info_df.loc[ft['fund_id'], 'index_id']
             try:
-                s = fsdf[ft['submit_date']][index_id][ft['fund_id']]
+                score_dic = fsdf[ft['submit_date']][index_id][self.score_select[index_id]] 
+                rank_list = sorted(score_dic.items(), key=lambda x:x[1], reverse=True)
+                rank_list = [i[0] for i in rank_list]
+                s = rank_list.index(ft['fund_id']) + 1
             except:
                 s = np.nan
             ft['submit_d_score'] = s
@@ -280,12 +289,12 @@ class ReportHelper:
         df = pd.DataFrame(ft_res)
         df['before_w'] = df['before_w'].round(4)
         df['after_w'] = df['after_w'].round(4)
-        dts = df.submit_date.unique().tolist()
-        for dt in dts:
-            _df = df[df['submit_date'] == dt]
-            fund_id = _df.iloc[-1].fund_id
-            df.loc[(df['submit_date'] == dt) & (df['fund_id'] == fund_id),'after_w'] = 1 - _df.iloc[:-1].after_w.sum()
-            df.loc[(df['submit_date'] == dt) & (df['fund_id'] == fund_id),'before_w'] = 1 - _df.iloc[:-1].before_w.sum()
+        # dts = df.submit_date.unique().tolist()
+        # for dt in dts:
+        #     _df = df[df['submit_date'] == dt]
+        #     fund_id = _df.iloc[-1].fund_id
+        #     df.loc[(df['submit_date'] == dt) & (df['fund_id'] == fund_id),'after_w'] = 1 - _df.iloc[:-1].after_w.sum()
+        #     df.loc[(df['submit_date'] == dt) & (df['fund_id'] == fund_id),'before_w'] = 1 - _df.iloc[:-1].before_w.sum()
         return df
 
     @staticmethod
@@ -465,8 +474,11 @@ class ReportHelper:
     def plot_taa_pct_df(self, dt):
         res = []
         _res = []
+        index_val_selected = list(TaaTunerParam.INDEX_PCT_SELECTED.keys())
         for index_id, pct_data in TaaTunerParam.POOL.items():
-            res.append( self.pct_df.loc[index_id][['datetime',pct_data]].reset_index().pivot_table(index='datetime', columns='index_id')[pct_data])
+            _df = self.pct_df.loc[index_id][['datetime'] + index_val_selected].rename(columns=TaaTunerParam.INDEX_PCT_SELECTED)
+            _df = _df[['datetime',pct_data]].reset_index().pivot_table(index='datetime', columns='index_id',values=pct_data)
+            res.append(_df)
             index_val_name = TaaTunerParam.PCT_PAIR[index_id]
             _df = self.pct_df.loc[index_id][['datetime',index_val_name]].reset_index()
             _df = _df.pivot_table(index='datetime',columns='index_id')[index_val_name]
@@ -633,7 +645,10 @@ class ReportHelper:
             _fund_list = list(weight_df[begin_date].keys())
             _nav = m.dts.fund_nav[_fund_list].loc[begin_date:today]
             fund_ret = _nav.iloc[-1] / _nav.iloc[0] - 1
-            ret_df = pd.DataFrame(fund_ret).rename(columns={0:'ret'})
+            ret_df = pd.DataFrame(fund_ret)
+            if 0 not in ret_df.columns:
+                return None
+            ret_df = ret_df.rename(columns={0:'ret'})
             res = []
             for k,w in weight_df[begin_date].items():
                 res.append({'fund_id':k,'weight':w})
@@ -653,6 +668,8 @@ class ReportHelper:
             today = end_date
         if begin_date is None:
             begin_date =  today - datetime.timedelta(days=today.weekday())
+        dt_df = m.dts.trading_days.copy().set_index('datetime')
+        begin_date = dt_df.loc[:begin_date].index[-1]
         trading_days = sorted(list(set(m.dts.trading_days.datetime) & set(mv_df.index)))
         mv_df = mv_df.loc[begin_date:].reindex(trading_days)
         stats = Calculator.get_stat_result_from_df(df=mv_df.dropna().reset_index(), date_column='date', value_column='mv')
@@ -679,7 +696,9 @@ class ReportHelper:
             for idx,dl in enumerate(date_list):
                 b1 = dl[0]
                 b2 = dl[1]
-                _res_dt_df.append(_ret_item(b1, b2, False, False))
+                fund_res = _ret_item(b1, b2, False, False)
+                if fund_res is not None:
+                    _res_dt_df.append(fund_res)
             res_df = pd.concat(_res_dt_df, axis=0, sort=True)
         #res_df.loc[:,'rets'] = res_df['ret'] * res_df['weight']
         res_df = res_df.sort_values('rets',ascending=False)
@@ -875,9 +894,6 @@ class ReportHelper:
             _df_before_agg = _df_before_agg.reset_index().drop_duplicates('subject').set_index('subject')  
             _df_before_agg = _df_before_agg.join(bond_info[['subject','is_city_bond']].sort_values('subject',ascending=False).drop_duplicates(subset=['subject']).set_index('subject'))
             _df_before_agg = _df_before_agg.join(bond_info[['subject','cbl2_type']].drop_duplicates(subset=['subject']).set_index('subject'))
-            
-
-
         else:
             _df_before_agg = _df_before_agg.groupby(f'{fund_type}_name').agg({'desc_name':lambda x: x.tolist(),f'{fund_type}_weight':'sum'})
 
@@ -933,10 +949,6 @@ class ReportHelper:
             result[f'{fund_type}_weight'] = result[f'{fund_type}_weight'].map(lambda x : str(round(100*x,1))+'%')
             return result, result2
 
-
-
-
-
     @staticmethod
     def port_penetraded_asset_weight(res_df, m, saa):
         res_df['weight'] = res_df.weight.map(lambda x: float(x[:-1]))
