@@ -703,17 +703,17 @@ class BasicFundTrader(Helper):
                     traded_list.append(trd)
         _trade_list_buy = [i  for i in trades if i.is_buy is True]
         _trade_list_sel = [i  for i in traded_list if i.is_buy is False]
-        plan_sel_amount = 0 if len(_trade_list_sel) < 1 else sum([i.amount for i in _trade_list_sel])
-        plan_buy_amount = 0 if len(_trade_list_buy) < 1 else sum([i.amount for i in _trade_list_buy])
-        if plan_sel_amount == 0 or plan_buy_amount == 0: 
-            buy_amount_adjusted_rate = 1
-        else:
-            buy_amount_adjusted_rate = plan_sel_amount / plan_buy_amount
+        # plan_sel_amount = 0 if len(_trade_list_sel) < 1 else sum([i.amount for i in _trade_list_sel])
+        # plan_buy_amount = 0 if len(_trade_list_buy) < 1 else sum([i.amount for i in _trade_list_buy])
+        # if plan_sel_amount == 0 or plan_buy_amount == 0:
+        #     buy_amount_adjusted_rate = 1
+        # else:
+        #     buy_amount_adjusted_rate = plan_sel_amount / plan_buy_amount
         
         for trd in trades:
             if not trd.is_buy:
                 continue
-            trd.amount *= buy_amount_adjusted_rate
+            # trd.amount *= buy_amount_adjusted_rate
             if self.ENABLE_COMMISSION:
                 purchase_fee = fund_purchase_fees[trd.fund_id] * self.PURCHASE_DISCOUNT 
                 if np.isnan(purchase_fee):
