@@ -3,6 +3,7 @@ import dataclasses
 import datetime
 import pandas
 import numpy as np
+import math
 import statsmodels.api as sm
 from statsmodels import regression
 from scipy.stats import gmean
@@ -16,6 +17,7 @@ from ..fund.engine.asset_helper import TAAHelper, TAAStatusMode
 from ..data.wrapper.mysql import *
 from ..data.view.basic_models import *
 from ..data.view.derived_models import *
+from ..data.manager.manager_fund import DataManager
 from .calculator_item import CalculatorBase
 
 @dataclasses.dataclass
@@ -63,6 +65,33 @@ class BenchmarkSeiesStatUnit(SeriesStatUnit):
     win_rate: float             # 胜率
     continue_value: float       # 业绩持续性
 
+@dataclasses.dataclass
+class RetStatUnit:
+
+    recent_natural_year_ret: float      # 今年以来收益 未年化
+    recent_1w_ret: float                # 最近一周收益 未年化
+    recent_1m_ret: float                # 最近一月收益 未年化
+    recent_3m_ret: float                # 最近三月收益 未年化
+    recent_6m_ret: float                # 最近半年收益 未年化
+    recent_1y_ret: float                # 最近一年收益 未年化
+    recent_3y_ret: float                # 最近三年收益 未年化
+    recent_5y_ret: float                # 最近五年收益 未年化
+    history_ret: float                  # 成立以来收益 未年化
+    history_annual_ret: float           # 成立以来收益 年化
+
+@dataclasses.dataclass
+class VolStatUnit:
+
+    recent_natural_year_vol: float      # 今年以来风险 年化
+    recent_1w_vol: float                # 最近一周风险 年化
+    recent_1m_vol: float                # 最近一月风险 年化
+    recent_3m_vol: float                # 最近三月风险 年化
+    recent_6m_vol: float                # 最近半年风险 年化
+    recent_1y_vol: float                # 最近一年风险 年化
+    recent_3y_vol: float                # 最近三年风险 年化
+    recent_5y_vol: float                # 最近五年风险 年化
+    history_vol: float                  # 成立以来风险 年化
+
 class Calculator:
 
     TRADING_DAYS_PER_YEAR = 242
@@ -170,6 +199,201 @@ class Calculator:
             continue_value=res['continue_value_m'],
         )
 
+    @staticmethod
+    def get_recent_ret(dates:             pandas.core.series.Series,
+                       values:            pandas.core.series.Series,
+                       frequency:         str='1D'):
+        res = CalculatorBase.calc_recent_ret(dates=dates,values=values,frequency=frequency)
+        return RetStatUnit(
+            recent_natural_year_ret=res['recent_natural_year_ret'],
+            recent_1w_ret=res['recent_1w_ret'],
+            recent_1m_ret=res['recent_1m_ret'],
+            recent_3m_ret=res['recent_3m_ret'],
+            recent_6m_ret=res['recent_6m_ret'],
+            recent_1y_ret=res['recent_1y_ret'],
+            recent_3y_ret=res['recent_3y_ret'],
+            recent_5y_ret=res['recent_5y_ret'],
+            history_ret=res['history_ret'],
+            history_annual_ret=res['history_annual_ret'],
+        )
+
+    @staticmethod
+    def get_recent_vol(dates:             pandas.core.series.Series,
+                       values:            pandas.core.series.Series,
+                       frequency:         str='1D'):
+        res = CalculatorBase.calc_recent_vol(dates=dates,values=values,frequency=frequency)
+        return VolStatUnit(
+            recent_natural_year_vol=res['recent_natural_year_vol'],
+            recent_1w_vol=res['recent_1w_vol'],
+            recent_1m_vol=res['recent_1m_vol'],
+            recent_3m_vol=res['recent_3m_vol'],
+            recent_6m_vol=res['recent_6m_vol'],
+            recent_1y_vol=res['recent_1y_vol'],
+            recent_3y_vol=res['recent_3y_vol'],
+            recent_5y_vol=res['recent_5y_vol'],
+            history_vol=res['history_vol'],
+        )
+
+    @staticmethod
+    def get_penetraded_stock_analysis(fund_list: pandas.core.series.Series,
+                                      weight_list: pandas.core.series.Series):
+        # fund_list 基金代码 比如 ['000017!0', '000020!0', '000031!0', '000051!0', '000061!0'] 形式
+        # weight_list 权重代码 比如 [0.1,0.2,0.2,0.2,0.3] 形式
+        df = pandas.DataFrame({'fund_id':fund_list,'weight':weight_list}).set_index('fund_id')
+        fund_info = DataManager.basic_data(func_name=f'get_fund_info', fund_list = fund_list.tolist())
+        fund_type = 'stock'
+        fund_list = df.index.tolist()
+        stock_weights = DataManager.basic_data(func_name=f'get_fund_hold_{fund_type}_latest', fund_list = fund_list)
+        stock_weights = stock_weights.set_index('fund_id').drop(columns=['datetime','_update_time'])
+        weight_list = [i for i in stock_weights if 'weight' in i]
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            fund_weight = df.loc[fund_id, 'weight'] /100
+            for weight_i in weight_list:
+                stock_weights.loc[fund_id, weight_i] = stock_weights.loc[fund_id, weight_i] / 100 * fund_weight
+        _df = stock_weights.join(df)
+        _res = []
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            for i in range(1,11):
+                dic = {
+                    'fund_id' : fund_id,
+                    f'{fund_type}_name':r[1][f'rank{i}_{fund_type}'],
+                    f'{fund_type}_weight':r[1][f'rank{i}_{fund_type}weight'], 
+                    f'{fund_type}_id':r[1][f'rank{i}_{fund_type}_code']
+                }
+                _res.append(dic)
+        df = pandas.DataFrame(_res)
+        df = pandas.merge(df,fund_info[['fund_id','desc_name']],on='fund_id')
+        _sum = df[f'{fund_type}_weight'].sum()
+        df[f'{fund_type}_weight'] = df[f'{fund_type}_weight'] / _sum
+        df = df.sort_values(f'{fund_type}_weight',ascending=False)
+
+        stock_list = df.stock_id.tolist()
+        stock_info = DataManager.raw_data(func_name=f'get_em_stock_info', stock_list=stock_list)
+        industry_info = DataManager.raw_data(func_name='get_em_industry_info',ind_class_type=IndClassType.SWI1)
+        stock_info.loc[:,'sw_indus_1'] = stock_info.bl_sws_ind_code.map(lambda x: x.split('-')[0] if x is not None else None)
+        _df1 = stock_info[['stock_id','name','sw_indus_1']]
+        _df2 = industry_info[['em_id','ind_name']].rename(columns={'em_id':'sw_indus_1'})
+        stock_info = pandas.merge(_df1,_df2,on='sw_indus_1')
+        df = df.join(stock_info[['stock_id','ind_name']].set_index('stock_id'),on='stock_id')
+        stock_price = DataManager.raw_data(func_name='get_em_stock_price_recent',codes=stock_list,columns=['pre_close'])
+        stock_price.loc[:,'ret'] =  stock_price.close / stock_price.pre_close - 1
+        df.join(stock_price[['stock_id','close','ret']].set_index('stock_id'), on='stock_id')
+        return df
+
+    @staticmethod
+    def group_left_year(x):
+        if x <= 1:
+            return '1年'
+        if x <= 3:
+            return '3年'
+        if x <= 5:
+            return '5年'
+        if x <= 10:
+            return '10年'
+
+    @staticmethod
+    def get_penetraded_bond_analysis(fund_list: pandas.core.series.Series,
+                                      weight_list: pandas.core.series.Series):
+        # fund_list 基金代码 比如 ['000005!0', '000015!0', '000016!0', '000032!0', '000033!0']  array 形式
+        # weight_list 权重代码 比如 [0.1,0.2,0.2,0.2,0.3] array 形式
+        df = pandas.DataFrame({'fund_id':fund_list,'weight':weight_list}).set_index('fund_id')
+        fund_info = DataManager.basic_data(func_name=f'get_fund_info', fund_list = fund_list.tolist())
+        fund_type = 'bond'
+        fund_list = df.index.tolist()
+        stock_weights = DataManager.basic_data(func_name=f'get_fund_hold_{fund_type}_latest', fund_list = fund_list)
+        stock_weights = stock_weights.set_index('fund_id').drop(columns=['datetime','_update_time'])
+        weight_list = [i for i in stock_weights if 'weight' in i]
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            fund_weight = df.loc[fund_id, 'weight'] /100
+            for weight_i in weight_list:
+                stock_weights.loc[fund_id, weight_i] = stock_weights.loc[fund_id, weight_i] / 100 * fund_weight
+        _df = stock_weights.join(df)
+        _res = []
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            for i in range(1,11):
+                dic = {
+                    'fund_id' : fund_id,
+                    f'{fund_type}_name':r[1][f'rank{i}_{fund_type}'],
+                    f'{fund_type}_weight':r[1][f'rank{i}_{fund_type}weight'], 
+                    f'{fund_type}_id':r[1][f'rank{i}_{fund_type}_code']
+                }
+                _res.append(dic)
+        df = pandas.DataFrame(_res)
+        df = pandas.merge(df,fund_info[['fund_id','desc_name']],on='fund_id')
+        df = df.dropna(subset=['bond_name'])
+        _bond_list = df.bond_id.tolist()
+        _bond_rate = DataManager.raw_data(func_name='get_em_bond_rate_maturity',codes=_bond_list)
+        df = df.set_index('bond_id').join(_bond_rate[['bond_id','maturity','subject_rate']].set_index('bond_id')).reset_index()
+        bond_info = DataManager.raw_data(func_name='get_em_bond_info',bond_id_list=_bond_list)
+        _bond_info_bond_list = bond_info.bond_id.tolist()
+        _conv_bond_list = [i for i in _bond_list if i not in _bond_info_bond_list]
+
+        conv_bond_info = DataManager.raw_data(func_name='get_em_conv_bond_info',bond_id_list=_conv_bond_list)
+        conv_bond_info = conv_bond_info[['CODES']].rename(columns={'CODES':'bond_id'})
+        conv_bond_info.loc[:,'cbl1_type'] = '可转债'
+        bond_info.loc[:,'cbl1_type'] = bond_info.cbl1_type.map(lambda x: x.replace('(中债)','').replace('（中债）',''))
+        df = df.join(bond_info[['bond_id','cbl1_type']].append(conv_bond_info[['bond_id','cbl1_type']]).set_index('bond_id'), on='bond_id')
+        _sum = df[f'{fund_type}_weight'].sum()
+        df[f'{fund_type}_weight'] = df[f'{fund_type}_weight'] / _sum
+        df = df.sort_values(f'{fund_type}_weight',ascending=False)
+        today = datetime.datetime.now().date()
+        td = (df.maturity - today)
+        _years = [_.days / 365 for _ in td]
+        df.loc[:,'maturity_yearly'] = _years
+        df.loc[:,'left_year'] = df.maturity_yearly.map(lambda x: Calculator.group_left_year(x))
+        bond_weight_df = df[['cbl1_type','bond_weight']].dropna().groupby('cbl1_type').sum()
+        bond_weight_df = bond_weight_df / bond_weight_df.sum()
+        bond_rate_df = df[['subject_rate','bond_weight']].dropna().groupby('subject_rate').sum()
+        bond_rate_df = bond_rate_df / bond_rate_df.sum()
+        left_maturity_year = df[['left_year','bond_weight']].dropna().groupby('left_year').sum()
+        left_maturity_year = left_maturity_year / left_maturity_year.sum()
+        result = {
+            'bond_in_funds': df, #基金组合持有债券穿透分析
+            'bond_type_weight': bond_weight_df, #基金组合债券种类分布
+            'bond_rate_weight': bond_rate_df, # 基金组合债券评级分布
+            'bond_maturity_weight': left_maturity_year, # 债券组合债券到期分布
+        }
+        return result
+
+    @staticmethod
+    def get_fund_last_week_last_trade_df(dt:datetime.date=None, fund_list:list=[]):
+        # 日期不填默认上周最后交易日开始
+        if dt is None:
+            dt = DataManager.basic_data(func_name='get_last_week_last_trade_dt')
+        fund_nav = DataManager.basic_data(func_name='get_fund_nav_with_date',start_date=dt,fund_list=fund_list)
+        fund_nav = fund_nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
+        fund_nav = pandas.DataFrame((fund_nav / fund_nav.iloc[0]).iloc[-1] - 1)
+        fund_nav.columns = ['ret']
+        return fund_nav
+
+    @staticmethod
+    def get_fund_annual_ret_vol_recent_and_history(fund_list:list, hold_date_list:list):
+        nav = DataManager.basic_data(func_name='get_fund_nav_with_date',fund_list=fund_list)
+        nav = nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
+        result = {}
+        for fund_id, hold_date in zip(fund_list,hold_date_list):
+            result[fund_id] = {}
+            for b_d in [None, hold_date]:
+                if b_d is None:
+                    nav_i = nav[fund_id].dropna()
+                else:
+                    nav_i = nav[fund_id].loc[b_d:].dropna()
+                last_unit_nav = nav_i.iloc[-1] / nav_i.iloc[0]
+                days = (nav_i.index[-1] - nav_i.index[0]).days
+                annual_ret = math.pow(last_unit_nav, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
+                annual_vol = nav_i.pct_change(1).std(ddof=1) * np.sqrt(CalculatorBase.TRADING_DAYS_PER_YEAR)
+                if b_d is None:
+                    result[fund_id]['history_annual_ret'] = annual_ret
+                    result[fund_id]['history_annual_vol'] = annual_vol
+                else:
+                    result[fund_id]['hold_annual_ret'] = annual_ret
+                    result[fund_id]['hold_annual_vol'] = annual_vol
+        return result
+
     @staticmethod
     def get_manager_result(dates: pandas.core.series.Series,
                           values: pandas.core.series.Series,
