@@ -108,6 +108,119 @@ class CalculatorBase:
         continue_regress_v = res.params[0]
         return continue_regress_v
 
+    @staticmethod
+    def calc_recent_ret(dates:             pd.core.series.Series,
+                        values:            pd.core.series.Series,
+                        frequency:         str='1D'):
+        assert frequency in ['1D'], f'frequency provided: {frequency} is not included, must in 1D'
+        dates = pd.to_datetime(dates)
+        dates = [i.date() for i in dates]
+        _end_dt = dates[-1]
+        last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
+        assert len(dates) == len(values), 'date and port_values has different numbers'
+        sr = pd.Series(values, index=dates).sort_index().dropna()
+        sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency).last().dropna()
+        sr.index = [i.date() for i in sr.index]
+        _begin_date = sr.index[0]
+        _natural_year_begin_date = datetime.date(last_date.year,1,1)
+        recent_natural_year_ret = sr.loc[:last_date][-1] / sr.loc[:max(_natural_year_begin_date,_begin_date)][-1] - 1
+        recent_1w_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['w1'],_begin_date)][-1] - 1
+        recent_1m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m1'],_begin_date)][-1] - 1
+        recent_3m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m3'],_begin_date)][-1] - 1
+        recent_6m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m6'],_begin_date)][-1] - 1
+        recent_1y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y1'],_begin_date)][-1] - 1
+        recent_3y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y3'],_begin_date)][-1] - 1
+        recent_5y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y5'],_begin_date)][-1] - 1
+        history_ret = sr.loc[:last_date][-1] / sr[0] - 1
+        days = (dates[-1] - dates[0]).days
+        history_annual_ret = math.pow(history_ret + 1, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
+        
+        return {
+            'recent_natural_year_ret':recent_natural_year_ret,
+            'recent_1w_ret':recent_1w_ret,
+            'recent_1m_ret':recent_1m_ret,
+            'recent_3m_ret':recent_3m_ret,
+            'recent_6m_ret':recent_6m_ret,
+            'recent_1y_ret':recent_1y_ret,
+            'recent_3y_ret':recent_3y_ret,
+            'recent_5y_ret':recent_5y_ret,
+            'history_ret':history_ret,
+            'history_annual_ret':history_annual_ret,
+        }
+
+    @staticmethod
+    def get_period_ret(dates: pd.core.series.Series,
+                       values: pd.core.series.Series,
+                       frequency: str='3M'):
+        assert frequency in ['1D','1W','2W','1M','3M','1Y'], 'frequency provided is not included, must in 1D 1W 2W 1M 3M 1Y'
+        sr = pd.Series(values, index=dates).sort_index().dropna()
+        sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency,).last().dropna()
+        df = pd.DataFrame(sr.pct_change(1).dropna())
+        df.columns = ['ret']
+        td = df.index
+        df.index = [i.date() for i in td]
+        return df
+
+    @staticmethod
+    def calc_recent_vol(dates:             pd.core.series.Series,
+                        values:            pd.core.series.Series,
+                        frequency:         str='1D'):
+        assert frequency in ['1D','1W'], 'frequency provided is not included, must in 1D 1W 2W 1M'
+        dates = pd.to_datetime(dates)
+        dates = [i.date() for i in dates]
+        _end_dt = dates[-1]
+        last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
+        assert len(dates) == len(values), 'date and port_values has different numbers'
+        sr = pd.Series(values, index=dates).sort_index().dropna()
+        sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency).last().dropna()
+        sr.index = [i.date() for i in sr.index]
+        sr_ret = sr.pct_change(1).dropna()
+        if frequency == '1D':    
+            period_1y = CalculatorBase.TRADING_DAYS_PER_YEAR
+        elif frequency == '1W':
+            period_1y = CalculatorBase.TOTAL_WEEKS_PER_YEAR
+        _begin_date = sr_ret.index[0]
+        _natural_year_begin_date = datetime.date(last_date.year,1,1)
+        recent_natural_year_vol = sr_ret.loc[_natural_year_begin_date:].std(ddof=1) * np.sqrt(period_1y)
+        recent_1w_vol = sr_ret.loc[max(begin_date_dic['w1'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_1m_vol = sr_ret.loc[max(begin_date_dic['m1'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_3m_vol = sr_ret.loc[max(begin_date_dic['m3'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_6m_vol = sr_ret.loc[max(begin_date_dic['m6'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_1y_vol = sr_ret.loc[max(begin_date_dic['y1'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_3y_vol = sr_ret.loc[max(begin_date_dic['y3'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        recent_5y_vol = sr_ret.loc[max(begin_date_dic['y5'],_begin_date):].std(ddof=1) * np.sqrt(period_1y)
+        history_vol = sr_ret.std(ddof=1) * np.sqrt(period_1y)
+        return {
+            'recent_natural_year_vol':recent_natural_year_vol,
+            'recent_1w_vol':recent_1w_vol,
+            'recent_1m_vol':recent_1m_vol,
+            'recent_3m_vol':recent_3m_vol,
+            'recent_6m_vol':recent_6m_vol,
+            'recent_1y_vol':recent_1y_vol,
+            'recent_3y_vol':recent_3y_vol,
+            'recent_5y_vol':recent_5y_vol,
+            'history_vol':history_vol,
+        }
+
+    @staticmethod
+    def get_period_vol(dates: pd.core.series.Series,
+                       values: pd.core.series.Series,
+                       frequency: str='3M'):
+        assert frequency in ['1D','1W','2W','1M','3M','1Y'], 'frequency provided is not included, must in 1D 1W 2W 1M 3M 1Y'
+        sr = pd.Series(values, index=dates).sort_index().dropna()
+        sr = sr.set_axis(pd.to_datetime(sr.index), inplace=False).resample(frequency,).std(ddof=1).dropna()
+        df = pd.DataFrame(sr) * np.sqrt(CalculatorBase.TRADING_DAYS_PER_YEAR)
+        df.columns = ['vol']
+        td = df.index
+        df.index = [i.date() for i in td]
+        return df
+
+    @staticmethod
+    def get_fund_correlation_df(df: pd.DataFrame):
+        # 列是资产代码 index是日期
+        # 求最近一年日度收益相关性
+        return df.pct_change(1).corr()
+
     @staticmethod
     def get_stat_result(dates:             pd.core.series.Series,
                         values:            pd.core.series.Series,
@@ -132,37 +245,29 @@ class CalculatorBase:
                 period_3m = period_1m * 3
                 period_6m = period_1m * 6
                 period_1y = CalculatorBase.TRADING_DAYS_PER_YEAR
-                period_3y = period_1y * 3
-                period_5y = period_1y * 5
                 risk_free_per_period = risk_free_rate / CalculatorBase.TRADING_DAYS_PER_YEAR
             elif frequency == '1W':
                 period_1m = 4
                 period_3m = period_1m * 3
                 period_6m = period_1m * 6
                 period_1y = CalculatorBase.TOTAL_WEEKS_PER_YEAR
-                period_3y = period_1y * 3
-                period_5y = period_1y * 5
                 risk_free_per_period = risk_free_rate / 52
             elif frequency == '2W':
                 period_1m = 2
                 period_3m = period_1m * 3
                 period_6m = period_1m * 6
                 period_1y = CalculatorBase.TOTAL_WEEKS_PER_YEAR / 2
-                period_3y = period_1y * 3
                 risk_free_per_period = risk_free_rate / 26
             elif frequency == '1M':
                 period_1m = 1
                 period_3m = period_1m * 3
                 period_6m = period_1m * 6
                 period_1y = CalculatorBase.TOTAL_MONTHS_PER_YEAR
-                period_3y = period_1y * 3
-                period_5y = period_1y * 5
                 risk_free_per_period = risk_free_rate / 12
             if ret_method == 'log_ret':
                 sr_ret = np.log(sr).diff(1).iloc[1:]
             elif ret_method == 'pct_ret':
                 sr_ret = sr.pct_change(1).iloc[1:]
-
             start_date = sr.index[0] 
             end_date = sr.index[-1]
             days = (dates[-1] - dates[0]).days
@@ -172,13 +277,14 @@ class CalculatorBase:
             annual_ret = math.pow(last_unit_nav, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
             annual_vol = sr_ret.std(ddof=1) * np.sqrt(period_1y)
             _begin_date = sr.index[0]
-            recent_1w_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['w1'],_begin_date)][-1] - 1
-            recent_1m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m1'],_begin_date)][-1] - 1
-            recent_3m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m3'],_begin_date)][-1] - 1
-            recent_6m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m6'],_begin_date)][-1] - 1
-            recent_1y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y1'],_begin_date)][-1] - 1
-            recent_3y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y3'],_begin_date)][-1] - 1
-            recent_5y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y5'],_begin_date)][-1] - 1
+            recent_ret_res = CalculatorBase.calc_recent_ret(dates=dates,values=values,frequency='1D')
+            recent_1w_ret = recent_ret_res['recent_1w_ret']
+            recent_1m_ret = recent_ret_res['recent_1m_ret']
+            recent_3m_ret = recent_ret_res['recent_3m_ret']
+            recent_6m_ret = recent_ret_res['recent_6m_ret']
+            recent_1y_ret = recent_ret_res['recent_1y_ret']
+            recent_3y_ret = recent_ret_res['recent_3y_ret']
+            recent_5y_ret = recent_ret_res['recent_5y_ret']
             worst_3m_ret = sr.pct_change(period_3m).min()
             worst_6m_ret = sr.pct_change(period_6m).min()
             last_mv_diff = sr[-1] - sr[-2]
@@ -265,15 +371,19 @@ class CalculatorBase:
                 up_capture = up_cap_top / up_cap_bot
 
                 df_ret_down = df_ret[df_ret.index_ret < 0] + 1
-                down_cap_top = df_ret_down.fund_ret.product() ** (period_1y/len(df_ret_down)) - 1
-                down_cap_bot = df_ret_down.index_ret.product() ** (period_1y/len(df_ret_down)) - 1
+                df_ret_down_length = max(1,len(df_ret_down))
+                down_cap_top = df_ret_down.fund_ret.product() ** (period_1y/df_ret_down_length) - 1
+                down_cap_bot = df_ret_down.index_ret.product() ** (period_1y/df_ret_down_length) - 1
                 down_capture = down_cap_top / down_cap_bot
 
                 df_ret['excess_ret'] = df_ret.fund_ret - df_ret.index_ret
                 df_ret = df_ret.replace(np.Inf,np.nan).replace(-np.Inf,np.nan).dropna()
-                mod = AutoReg(endog=df_ret.excess_ret.values, lags=1)
-                res = mod.fit()
-                excess_ret_cotinue = res.params[0]
+                if df_ret.shape[0] < 4:
+                    excess_ret_cotinue = np.nan
+                else:
+                    mod = AutoReg(endog=df_ret.excess_ret.values, lags=1)
+                    res = mod.fit()
+                    excess_ret_cotinue = res.params[0]
                 return {
                     'start_date':start_date,
                     'end_date':end_date,
