@@ -87,13 +87,17 @@ class WechatBot(object):
 
     def send_hedge_fund_nav_update_failed(self, err_msg: str):
         markdown_content = '#### <font color=\"warning\">更新私募基金净值数据错误！！</font>'
-        markdown_content += f'\n>err msg: {err_msg}'
+        markdown_content += f'\n>**err msg:** {err_msg}'
         self.send(markdown_content)
 
-    def send_fof_nav_update(self):
+    def send_fof_nav_update(self, df):
+        markdown_content = f'#### <font color=\"info\">FOF净值数据更新</font>'
+        markdown_content += f'\n>**{df.to_markdown()}**'
         self.send(markdown_content)
 
-    def send_fof_nav_update_failed(self, err_msg: str):
+    def send_fof_nav_update_failed(self, fof_id: str, err_msg: str):
+        markdown_content = f'#### <font color=\"info\">{fof_id}净值数据更新失败！！</font>'
+        markdown_content += f'\n>**err msg:** {err_msg}'
         self.send(markdown_content)
 
     def send_refinancing_key_info(self, dt, stocks, key):
